%%% Filename : p_encoding.pl
%%% Date     : 09.10.2013
%%% Contents : module for encoding the generated calculus in ISABELLE
%%% Author   : Lara Katharina Spendier <lara@logic.at>

%% encodingOut(+Axiom, +Rules, +BC)
%%	Axiom 	    ... Axiom the user provided
%% 	Rules  	    ... Analytic Rules for the axiom ([[Premises, Conclusion],...])
%%	BC 	    ... Base Calculus: 0 = LK+, 2 = BK
encodingOut(Axiom, Rules, BC) :-
	createEncoding(Rules, 1, EncodingRules, EncodingRuleNames),
	encodingToFile(Axiom, EncodingRules, BC, EncodingRuleNames).

encodingOut(100,_,_) :-
	tell('pencoding/ParalyzerEncoding.thy'),
	write('The encoding is not available.'),
	told.

%% createEncoding(+[Rules], +N, -[EncodingRules], -EncodingRuleName)
%% 	[Rules]  ... Logical rules generated by the algorithm
%%	N	 ... counter for the number of the rules
%% 	[EncodingRules]  ... Isabelle encoding for the logical rules
%%	EncodingRuleNames ... text string containing the rule names in the encoding
createEncoding([],_,[],'').
createEncoding([[Prem, Conc]|T],N,Encoding,EncodingRuleNames) :-
	%writef('\n Prem: %p / Con: %p', [Prem, Conc]),
	length(Prem, PSize),
	[_, SConc] = Conc,
	( SConc = [] ->
	  SConcT = 0
	; SConcT = 1
	),
	%writef('\n PSize: %p -- Ante %p - Succ %p -- N: %p', [PSize, AConc, SConc, N]),
	string_concat('ruleStar', N, Text1),
	string_concat(' ruleStar', N, ERN1),	
	string_concat(Text1, ': "', Text2),
	createSeqEncoding(Prem, SConcT, PremEnc),
	( PSize > 1 ->
	  string_concat(Text2, '[| ', Text3),
	  string_concat(Text3, PremEnc, Text4),
	  string_concat(Text4, '|] ==> ', Text5)
	; string_concat(Text2, PremEnc, Text3),
	  string_concat(Text3, ' ==> ', Text5)
	),
	createSeqEncoding([Conc], SConcT, ConcEnc),
	string_concat(Text5, ConcEnc, Text6),
	string_concat(Text6, '"\n', Text),
	N1 is N+1,
	createEncoding(T,N1,Encoding1,EncodingRuleNames1),
	(Encoding1 = [] ->
	  Encoding = Text,	
	  EncodingRuleNames = ERN1
	; string_concat(Text, Encoding1, Encoding),
	  string_concat(ERN1, EncodingRuleNames1, EncodingRuleNames)
	).

%% createSeqEncoding(+[Rules], +FormularIntroducedRight, -[EncodingRules])
%% 	[Rules]  ... Logical rules generated by the algorithm
%%	FormulaIntroducedRight ... 0 if formula is introduced on the left (=two sequences 
%%				on the left needed) or 1 if it is introduced on the right
%% 	[EncodingRules]  ... Isabelle encoding for the logical rules
createSeqEncoding([], _, '').
createSeqEncoding([[Ante,Succ]|T], 0, PremEnc) :-
% the formula is introduced on the left side ==> $G, $H
	createSeqEncoding(T, 0, PremEnc1),	
	%writef('\n PremEnc1: %p -- Ante: %p -- Succ: %p', [PremEnc1, Ante, Succ]),
	( Ante = [] ->  % if Ante is empty, then there has to be something in Succ and vice versa
	  getVar(Succ, Var),
	  string_concat('$G, $H |- $E, ', Var, PremEncT2)
	;
	  getVar(Ante, Var),
	  string_concat('$G,', Var, PremEncT1),
	  string_concat(PremEncT1, ',$H |- $E', PremEncT2)
	), 
	PremEnc2 = PremEncT2,
	%writef('\n PremEnc2: %p ', [PremEnc2]),
	( PremEnc1 \= '' ->
	  string_concat(PremEnc2, '; ', PremEnc11),
	  string_concat(PremEnc11, PremEnc1, PremEnc)
	; PremEnc = PremEnc2
	).
createSeqEncoding([[Ante,Succ]|T], 1, PremEnc) :-
% the formula is introduced on the right side ==> $E, $F
	createSeqEncoding(T, 1, PremEnc1),	
	%writef('\n PremEnc1: %p -- Ante: %p -- Succ: %p', [PremEnc1, Ante, Succ]),
	( Ante = [] ->
	  getVar(Succ, Var),
	  string_concat('$G |- $E, ', Var, PremEncT1),
 	  string_concat(PremEncT1, ',$F', PremEncT2)	
	;
	  getVar(Ante, Var),
	  string_concat('$G,', Var, PremEncT1),
  	  string_concat(PremEncT1, ' |- $E, $F', PremEncT2)
	), 
	PremEnc2 = PremEncT2,
	%writef('\n PremEnc2: %p ', [PremEnc2]),
	( PremEnc1 \= '' ->
	  string_concat(PremEnc2, '; ', PremEnc11),
	  string_concat(PremEnc11, PremEnc1, PremEnc)
	; PremEnc = PremEnc2
	).

%% getVar(+Formula, -FormulaEncoded)
%% 	Formula ... formula occurring in the rule
%%	FormulaEncoded ... formula in the correct output format
%%			   *1 = ~, *2 = +, v = |, -> = -->, a = P, b = Q
getVar(['*1' Var], VarR) :-
	getVar([Var], VarR1),
	string_concat('~', VarR1, VarR).
getVar(['*2' Var], VarR) :-
	getVar([Var], VarR1),
	string_concat('+', VarR1, VarR).
getVar([A v B], VarR) :-
	getVar([A], VarR1),
	getVar([B], VarR2),
	string_concat('(', VarR1, VarR3),
	string_concat(VarR3, '|', VarR4),
	string_concat(VarR4, VarR2, VarR5),
	string_concat(VarR5, ')', VarR).
getVar([A & B], VarR) :-
	getVar([A], VarR1),
	getVar([B], VarR2),
	string_concat('(', VarR1, VarR3),
	string_concat(VarR3, '&', VarR4),
	string_concat(VarR4, VarR2, VarR5),
	string_concat(VarR5, ')', VarR).
getVar([A -> B], VarR) :-
	getVar([A], VarR1),
	getVar([B], VarR2),
	string_concat('(', VarR1, VarR3),
	string_concat(VarR3, '-->', VarR4),
	string_concat(VarR4, VarR2, VarR5),
	string_concat(VarR5, ')', VarR).
getVar(['a'], 'P').
getVar(['b'], 'Q').


%% encodingToFile(+Axiom,+Rules,+BC,+ERN)
%%	writes the encoding in the file ParalzyerEncoding
%%	Axiom ... axiom(s) that was(were) transformed
%%	Rules ... generated rules in encoding format
%%	BC    ... base calculus; =0 if base calculus is LK+, else BK
%%	ERN   ... the names of the encoded rules
encodingToFile(Axiom,Rules,BC, ERN) :-
	telling(Old),
	tell('ptex/ParalyzerEncoding.thy'),
%	tell('ptex/ParalyzerEncoding.sty'),  
	writef("(*  Title:      Sequents/ParalyzerEncoding.thy \nAuthor:     Paralyzer, Vienna University of Technology\n            based on LK0.thy by L. Paulson, University of Cambridge\nCopyright   2013 Vienna University of Technology\n*)\n\nheader {* Propositonal Paraconsistent Logic"),	
	(BC = 0 ->
	  writef(" HCL+ + \n")
	; writef(" BK + \n")
	),
	writef('(%p) *}\n\ntheory ParalyzerEncoding\nimports Sequents\nbegin\n\nconsts\n  ', [Axiom]),
	writef('Trueprop       :: "two_seqi"\n  conj         :: "[o,o] => o"       (infixr "&" 35)\n  disj         :: "[o,o] => o"       (infixr "|" 30)\n  imp          :: "[o,o] => o"       (infixr "-->" 25)\n  not          :: "o => o"           ("~ _" [40] 40)\n  circ         :: "o => o"           ("+ _" [39] 39)\n\nsyntax\n "_Trueprop"    :: "two_seqe" ("((_)/ |- (_))" [6,6] 5)\n\nparse_translation {* [(@{syntax_const "_Trueprop"},\n   two_seq_tr @{const_syntax Trueprop})] *}\nprint_translation {* [(@{const_syntax Trueprop},\n   two_seq_tr\' @{syntax_const "_Trueprop"})] *}\n\nnotation (xsymbols)\n  conj  (infixr "\<and>" 35) and\n  disj  (infixr "\<or>" 30) and\n  imp  (infixr "\<longrightarrow>" 25) \n  and not ("\<not> _" [40] 40)\n  and circ ("\<circ> _" [39] 39)\n\nnotation (HTML output)\n  conj  (infixr "\<and>" 35) and\n  disj  (infixr "\<or>" 30)\n  and not ("\<not> _" [40] 40)\n  and circ ("\<circ> _" [39] 39)\n\naxioms\n\n  (*Structural rules: contraction, thinning, exchange*)\n\n  contRS: "$H |- $E, $S, $S, $F ==> $H |- $E, $S, $F"\n  contLS: "$H, $S, $S, $G |- $E ==> $H, $S, $G |- $E"\n\n  thinRS: "$H |- $E, $F ==> $H |- $E, $S, $F"\n  thinLS: "$H, $G |- $E ==> $H, $S, $G |- $E"\n\n  exchRS: "$H |- $E, $R, $S, $F ==> $H |- $E, $S, $R, $F"\n  exchLS: "$H, $R, $S, $G |- $E ==> $H, $S, $R, $G |- $E"\n\n  cut:   "[| $H |- $E, P;  $H, P |- $E |] ==> $H |- $E"\n\n  (*Propositional rules*)\n\n  basic: "$H, P, $G |- $E, P, $F"\n\n  conjR: "[| $H|- $E, P, $F;  $H|- $E, Q, $F |] ==> $H|- $E, P&Q, $F"\n  conjL: "$H, P, Q, $G |- $E ==> $H, P & Q, $G |- $E"\n\n  disjR: "$H |- $E, P, Q, $F ==> $H |- $E, P|Q, $F"\n  disjL: "[| $H, P, $G |- $E;  $H, Q, $G |- $E |] ==> $H, P|Q, $G |- $E"\n\n  impR:  "$H, P |- $E, Q, $F ==> $H |- $E, P-->Q, $F"\n  impL:  "[| $H,$G |- $E,P;  $H, Q, $G |- $E |] ==> $H, P-->Q, $G |- $E"\n\n  (** new **)\n  '),
	(BC = 0 ->
	  writef('  %p',[Rules])
	; writef('  notR:  "$H, P |- $E, $F ==> $H |- $E, ~P, $F"\n  circR: "$H, P, ~P |- $E, $F ==> $H |- $E, +P, $F"\n  circL: "[| $H, $G |- $E, P; $H, $G |- $E, ~P |] ==> $H, +P, $G |- $E"\n\n%p',[Rules])
	),
	writef('\n\n(** Structural Rules on formulas **)\n\n(*contraction*)\n\nlemma contR: "$H |- $E, P, P, $F ==> $H |- $E, P, $F"\n  by (rule contRS)\n\nlemma contL: "$H, P, P, $G |- $E ==> $H, P, $G |- $E"\n  by (rule contLS)\n\n(*thinning*)\n\nlemma thinR: "$H |- $E, $F ==> $H |- $E, P, $F"\n  by (rule thinRS)\n\nlemma thinL: "$H, $G |- $E ==> $H, P, $G |- $E"\n  by (rule thinLS)\n\n(*exchange*)\n\nlemma exchR: "$H |- $E, Q, P, $F ==> $H |- $E, P, Q, $F"\n  by (rule exchRS)\n\nlemma exchL: "$H, Q, P, $G |- $E ==> $H, P, Q, $G |- $E"\n  by (rule exchLS)\n\n(*Tactic*)\n\nML {*\n    val apply_tac =\n    let\n      val rules = @{thms basic  conjL conjR disjL disjR impL impR \n      '),
	( BC = 0 ->
	  writef(' %p', [ERN])
	; writef(' notR circR circL %p',[ERN])
	),
	writef('}\n    in\n      atac\n      ORELSE\' resolve_tac rules\n    end\n*}\nend'),
	told,
	tell(Old).

