%%% Filename : p_grammar.pl
%%% Date     : 03.04.2012
%%% Contents : DCG grammar for tex-output and formula-check
%%% Author   : Lara Katharina Spendier <lara@logic.at>

bs        	  --> [92].
eol 	  	  --> [13,10].

%% Definitions for the style-file

texNewAxiom(C)		--> bs, "def", bs, "NewAxiom", "{$", C, "$}", eol.
texNewAxiomNotMath(C)	--> bs, "def", bs, "NewAxiomNotMath", "{", C, "}", eol.
texNewRuleStart   	--> bs, "def", bs, "NewRule", "{".
texNewRuleEnd	  	--> "}", eol.
texAxiomClass(C)  	--> bs, "def", bs, "AxiomClass", "{$", bs, "mathcal", C, "$}", eol.
texNameBaseCalculus(C) 	--> bs, "def", bs, "NameBaseCalculus{",C,"}",eol.
texNameBaseCalculusP(C)	--> bs, "def", bs, "NameBaseCalculusP{",C,"}",eol.
texBaseCalculus(C) 	--> bs, "def", bs, "BaseCalculus{",bs,C,"}",eol.
texEquivalences(C) 	--> bs, "def", bs, "Equivalences{",bs,C,"}",eol.
texWrittenBC(C)	   	--> bs, "def", bs, "WrittenBC{",C,"}",eol.
texRule(C)		--> bs, "def", bs, "WRule{",C,"}",eol.

%% tex-rewriting of the (hyper)sequent rules with >= 0 premises

texNewAxC(C)      --> bs, "AxiomC{$",C,"$}",eol.
texNewUnC(C)	  --> bs, "UnaryInfC{$",C,"$}",eol.
texNoLine	  --> bs, "noLine", eol.

texNewUnCon(C)    --> bs, "UnaryInfC{$",C,"$}",bs,"DisplayProof",bs,"hspace*{0.3cm}",bs,"vspace*{0.3cm}",eol.
texNewBinCon(C)   --> bs, "BinaryInfC{$",C,"$}",bs,"DisplayProof",bs,"hspace*{0.3cm}",bs,"vspace*{0.3cm}",eol.
texNewTriCon(C)   --> bs, "TrinaryInfC{$",C,"$}",bs,"DisplayProof",bs,"hspace*{0.3cm}",bs,"vspace*{0.3cm}",eol.

rule2tex([[H|T],[H1|T1]])  --> rule2tex([H|T]),bs,"Ra",rule2tex([H1|T1]).
rule2tex('G')     --> bs,"Gamma".
rule2tex('D')     --> bs,"Delta".
rule2tex('')      --> "".
rule2tex([])	  --> "".
rule2tex(X)	  --> axiom2texC(X).
rule2tex(X)	  --> axiom2tex(X).
rule2tex([H|[]])  --> rule2tex(H).
rule2tex([H|T])   --> rule2tex(H), ",", rule2tex(T).

nr2tex(N)	  --> {number_codes(N, Cs)}, Cs.

%% axiom2tex  ... start of grammar S
%% axiom2texX ... subformula that contains the logical formula of the form: oo x, o x, o(x _ y)
%% axiom2texY ... subformula that contains no nested unary connectives
%% axiom2tex1 ... subformula that is already bound by one unary connective (to keep track)
%% axiom2texC ... subformula that is an atom

axiom2tex(X)	  --> axiom2texS(X).

axiom2texS(-X) 	   --> " ", bs, "lnot ", axiom2texC(X).
axiom2texS(-X) 	   --> " ", bs, "lnot ", axiom2texUC(X).
axiom2texS(-X) 	   --> " ", bs, "lnot (", axiom2texBC(X), ")".
axiom2texS('o'X)   --> " ", bs, "circ ", axiom2texC(X).
axiom2texS('o'X)   --> " ", bs, "circ ", axiom2texUC(X).
axiom2texS('o'X)   --> " ", bs, "circ (", axiom2texBC(X), ")".
axiom2texS('*1'X)  --> " ", bs, "star_1 ", axiom2texC(X).
axiom2texS('*1'X)  --> " ", bs, "star_1 ", axiom2texUC(X).
axiom2texS('*1'X)  --> " ", bs, "star_1 (", axiom2texBC(X), ")".
axiom2texS('*2'X)  --> " ", bs, "star_2 ", axiom2texC(X).
axiom2texS('*2'X)  --> " ", bs, "star_2 ", axiom2texUC(X).
axiom2texS('*2'X)  --> " ", bs, "star_2 (", axiom2texBC(X), ")".
axiom2texS(X & Y)  --> axiom2texRPa(X), " ", bs, "land ", axiom2texP1a(Y).
axiom2texS(X & Y)  --> axiom2texP1a(X), " ", bs, "land ", axiom2texRPa(Y).
axiom2texS(X & Y)  --> axiom2texRPb(X), " ", bs, "land ", axiom2texP1b(Y).
axiom2texS(X & Y)  --> axiom2texP1b(X), " ", bs, "land ", axiom2texRPb(Y).
axiom2texS(X & Y)  --> axiom2texR1a(X), " ", bs, "land ", axiom2texP1a(Y).
axiom2texS(X & Y)  --> axiom2texP1a(X), " ", bs, "land ", axiom2texR1a(Y).
axiom2texS(X & Y)  --> axiom2texR1b(X), " ", bs, "land ", axiom2texP1b(Y).
axiom2texS(X & Y)  --> axiom2texP1b(X), " ", bs, "land ", axiom2texR1b(Y).
axiom2texS(X & Y)  --> axiom2texR2(X), " ", bs, "land ", axiom2texP2(Y).
axiom2texS(X & Y)  --> axiom2texP2(X), " ", bs, "land ", axiom2texR2(Y).
axiom2texS(X v Y)  --> axiom2texRPa(X), " ", bs, "lor ", axiom2texP1a(Y).
axiom2texS(X v Y)  --> axiom2texP1a(X), " ", bs, "lor ", axiom2texRPa(Y).
axiom2texS(X v Y)  --> axiom2texRPb(X), " ", bs, "lor ", axiom2texP1b(Y).
axiom2texS(X v Y)  --> axiom2texP1b(X), " ", bs, "lor ", axiom2texRPb(Y).
axiom2texS(X v Y)  --> axiom2texR1a(X), " ", bs, "lor ", axiom2texP1a(Y).
axiom2texS(X v Y)  --> axiom2texP1a(X), " ", bs, "lor ", axiom2texR1a(Y).
axiom2texS(X v Y)  --> axiom2texR1b(X), " ", bs, "lor ", axiom2texP1b(Y).
axiom2texS(X v Y)  --> axiom2texP1b(X), " ", bs, "lor ", axiom2texR1b(Y).
axiom2texS(X v Y)  --> axiom2texR2(X), " ", bs, "lor ", axiom2texP2(Y).
axiom2texS(X v Y)  --> axiom2texP2(X), " ", bs, "lor ", axiom2texR2(Y).
axiom2texS(X -> Y)  --> axiom2texRPa(X), " ", bs, "ra ", axiom2texP1a(Y).
axiom2texS(X -> Y)  --> axiom2texP1a(X), " ", bs, "ra ", axiom2texRPa(Y).
axiom2texS(X -> Y)  --> axiom2texRPb(X), " ", bs, "ra ", axiom2texP1b(Y).
axiom2texS(X -> Y)  --> axiom2texP1b(X), " ", bs, "ra ", axiom2texRPb(Y).
axiom2texS(X -> Y)  --> axiom2texR1a(X), " ", bs, "ra ", axiom2texP1a(Y).
axiom2texS(X -> Y)  --> axiom2texP1a(X), " ", bs, "ra ", axiom2texR1a(Y).
axiom2texS(X -> Y)  --> axiom2texR1b(X), " ", bs, "ra ", axiom2texP1b(Y).
axiom2texS(X -> Y)  --> axiom2texP1b(X), " ", bs, "ra ", axiom2texR1b(Y).
axiom2texS(X -> Y)  --> axiom2texR2(X), " ", bs, "ra ", axiom2texP2(Y).
axiom2texS(X -> Y)  --> axiom2texP2(X), " ", bs, "ra ", axiom2texR2(Y).

%axiom2tex(X)	  --> axiom2texC(X).

axiom2texRPa(-X)    --> " ", bs, "lnot ", axiom2texCa(X).
axiom2texRPa('o'X)   --> " ", bs, "circ ", axiom2texCa(X).
axiom2texRPa('*1'X)  --> " ", bs, "star_1 ", axiom2texCa(X).
axiom2texRPa('*2'X)  --> " ", bs, "star_2 ", axiom2texCa(X).
axiom2texRPa(X & Y)  --> axiom2texRPa(X), " ", bs, "land ", axiom2texP1a(Y).
axiom2texRPa(X & Y)  --> axiom2texP1a(X), " ", bs, "land ", axiom2texRPa(Y).
axiom2texRPa(X v Y)  --> axiom2texRPa(X), " ", bs, "lor ", axiom2texP1a(Y).
axiom2texRPa(X v Y)  --> axiom2texP1a(X), " ", bs, "lor ", axiom2texRPa(Y).
axiom2texRPa(X -> Y)  --> axiom2texRPa(X), " ", bs, "ra ", axiom2texP1a(Y).
axiom2texRPa(X -> Y)  --> axiom2texP1a(X), " ", bs, "ra ", axiom2texRPa(Y).
axiom2texRPb(-X)    --> " ", bs, "lnot ", axiom2texCb(X).
axiom2texRPb('o'X)   --> " ", bs, "circ ", axiom2texCb(X).
axiom2texRPb('*1'X)  --> " ", bs, "star_1 ", axiom2texCb(X).
axiom2texRPb('*2'X)  --> " ", bs, "star_2 ", axiom2texCb(X).
axiom2texRPb(X & Y)  --> axiom2texRPb(X), " ", bs, "land ", axiom2texP1b(Y).
axiom2texRPb(X & Y)  --> axiom2texP1b(X), " ", bs, "land ", axiom2texRPb(Y).
axiom2texRPb(X v Y)  --> axiom2texRPb(X), " ", bs, "lor ", axiom2texP1b(Y).
axiom2texRPb(X v Y)  --> axiom2texP1b(X), " ", bs, "lor ", axiom2texRPb(Y).
axiom2texRPb(X -> Y)  --> axiom2texRPb(X), " ", bs, "ra ", axiom2texP1b(Y).
axiom2texRPb(X -> Y)  --> axiom2texP1b(X), " ", bs, "ra ", axiom2texRPb(Y).


axiom2texR1a(-X)     --> " ", bs, "lnot ", axiom2texUCa(X).
axiom2texR1a('o'X)   --> " ", bs, "circ ", axiom2texUCa(X).
axiom2texR1a('*1'X)  --> " ", bs, "star_1 ", axiom2texUCa(X).
axiom2texR1a('*2'X)  --> " ", bs, "star_2 ", axiom2texUCa(X).
axiom2texR1a(X & Y)  --> axiom2texR1a(X), " ", bs, "land ", axiom2texP1a(Y).
axiom2texR1a(X & Y)  --> axiom2texP1a(X), " ", bs, "land ", axiom2texR1a(Y).
axiom2texR1a(X v Y)  --> axiom2texR1a(X), " ", bs, "lor ", axiom2texP1a(Y).
axiom2texR1a(X v Y)  --> axiom2texP1a(X), " ", bs, "lor ", axiom2texR1a(Y).
axiom2texR1a(X -> Y)  --> axiom2texR1a(X), " ", bs, "ra ", axiom2texP1a(Y).
axiom2texR1a(X -> Y)  --> axiom2texP1a(X), " ", bs, "ra ", axiom2texR1a(Y).
axiom2texR1b(-X)    --> " ", bs, "lnot ", axiom2texUCb(X).
axiom2texR1b('o'X)   --> " ", bs, "circ ", axiom2texUCb(X).
axiom2texR1b('*1'X)  --> " ", bs, "star_1 ", axiom2texUCb(X).
axiom2texR1b('*2'X)  --> " ", bs, "star_2 ", axiom2texUCb(X).
axiom2texR1b(X & Y)  --> axiom2texR1b(X), " ", bs, "land ", axiom2texP1b(Y).
axiom2texR1b(X & Y)  --> axiom2texP1b(X), " ", bs, "land ", axiom2texR1b(Y).
axiom2texR1b(X v Y)  --> axiom2texR1b(X), " ", bs, "lor ", axiom2texP1b(Y).
axiom2texR1b(X v Y)  --> axiom2texP1b(X), " ", bs, "lor ", axiom2texR1b(Y).
axiom2texR1b(X -> Y)  --> axiom2texR1b(X), " ", bs, "ra ", axiom2texP1b(Y).
axiom2texR1b(X -> Y)  --> axiom2texP1b(X), " ", bs, "ra ", axiom2texR1b(Y).

axiom2texP1a(X)	     --> axiom2texC(X).
axiom2texP1a(-X)     --> " ", bs, "lnot ", axiom2texCa(X).
axiom2texP1a('o'X)   --> " ", bs, "circ ", axiom2texCa(X).
axiom2texP1a('*1'X)  --> " ", bs, "star_1 ", axiom2texCa(X).
axiom2texP1a('*2'X)  --> " ", bs, "star_2 ", axiom2texCa(X).
axiom2texP1a(X & Y)  --> axiom2texP1a(X), " ", bs, "land ", axiom2texP1a(Y).
axiom2texP1a(X & Y)  --> axiom2texP1a(X), " ", bs, "land ", axiom2texP1a(Y).
axiom2texP1a(X v Y)  --> axiom2texP1a(X), " ", bs, "lor ", axiom2texP1a(Y).
axiom2texP1a(X v Y)  --> axiom2texP1a(X), " ", bs, "lor ", axiom2texP1a(Y).
axiom2texP1a(X -> Y) --> axiom2texP1a(X), " ", bs, "ra ", axiom2texP1a(Y).
axiom2texP1a(X -> Y) --> axiom2texP1a(X), " ", bs, "ra ", axiom2texP1a(Y).
axiom2texP1b(X)	     --> axiom2texC(X).
axiom2texP1b(-X)     --> " ", bs, "lnot ", axiom2texCb(X).
axiom2texP1b('o'X)   --> " ", bs, "circ ", axiom2texCb(X).
axiom2texP1b('*1'X)  --> " ", bs, "star_1 ", axiom2texCb(X).
axiom2texP1b('*2'X)  --> " ", bs, "star_2 ", axiom2texCb(X).
axiom2texP1b(X & Y)  --> axiom2texP1b(X), " ", bs, "land ", axiom2texP1b(Y).
axiom2texP1b(X & Y)  --> axiom2texP1b(X), " ", bs, "land ", axiom2texP1b(Y).
axiom2texP1b(X v Y)  --> axiom2texP1b(X), " ", bs, "lor ", axiom2texP1b(Y).
axiom2texP1b(X v Y)  --> axiom2texP1b(X), " ", bs, "lor ", axiom2texP1b(Y).
axiom2texP1b(X -> Y) --> axiom2texP1b(X), " ", bs, "ra ", axiom2texP1b(Y).
axiom2texP1b(X -> Y) --> axiom2texP1b(X), " ", bs, "ra ", axiom2texP1b(Y).

axiom2texR2(-X)	    --> " ", bs, "lnot ", axiom2texBC(X).
axiom2texR2('o'X)   --> " ", bs, "circ ", axiom2texBC(X).
axiom2texR2('*1'X)  --> " ", bs, "star_1 ", axiom2texBC(X).
axiom2texR2('*2'X)  --> " ", bs, "star_2 ", axiom2texBC(X).
axiom2texR2(X & Y)  --> axiom2texR2(X), " ", bs, "land ", axiom2texP2(Y).
axiom2texR2(X & Y)  --> axiom2texP2(X), " ", bs, "land ", axiom2texR2(Y).
axiom2texR2(X v Y)  --> axiom2texR2(X), " ", bs, "lor ", axiom2texP2(Y).
axiom2texR2(X v Y)  --> axiom2texP2(X), " ", bs, "lor ", axiom2texR2(Y).
axiom2texR2(X -> Y)  --> axiom2texR2(X), " ", bs, "ra ", axiom2texP2(Y).
axiom2texR2(X -> Y)  --> axiom2texP2(X), " ", bs, "ra ", axiom2texR2(Y).

axiom2texP2(X)	    --> axiom2texC(X).
axiom2texP2(-X)	    --> " ", bs, "lnot ", axiom2texC(X).
axiom2texP2('o'X)   --> " ", bs, "circ ", axiom2texC(X).
axiom2texP2('*1'X)  --> " ", bs, "star_1 ", axiom2texC(X).
axiom2texP2('*2'X)  --> " ", bs, "star_2 ", axiom2texC(X).
axiom2texP2(X & Y)  --> axiom2texP2(X), " ", bs, "land ", axiom2texP2(Y).
axiom2texP2(X & Y)  --> axiom2texP2(X), " ", bs, "land ", axiom2texP2(Y).
axiom2texP2(X v Y)  --> axiom2texP2(X), " ", bs, "lor ", axiom2texP2(Y).
axiom2texP2(X v Y)  --> axiom2texP2(X), " ", bs, "lor ", axiom2texP2(Y).
axiom2texP2(X -> Y)  --> axiom2texP2(X), " ", bs, "ra ", axiom2texP2(Y).
axiom2texP2(X -> Y)  --> axiom2texP2(X), " ", bs, "ra ", axiom2texP2(Y).

axiom2texC(a) 	    --> bs,"alpha".
axiom2texC(b) 	    --> bs,"beta".
axiom2texCa(a) 	    --> bs,"alpha".
axiom2texCb(b) 	    --> bs,"beta".
axiom2texUC(-X)	    --> " ", bs,"lnot ", axiom2texC(X).
axiom2texUC('o'X)   --> " ", bs, "circ ", axiom2texC(X).
axiom2texUC('*1'X)  --> " ", bs, "star_1 ", axiom2texC(X).
axiom2texUC('*2'X)  --> " ", bs, "star_2 ", axiom2texC(X).
axiom2texUCa(-a)    --> " ", bs,"lnot ", bs, "alpha".
axiom2texUCa('o'a)  --> " ", bs, "circ ", bs, "alpha".
axiom2texUCa('*1'a) --> " ", bs, "star_1 ", bs, "alpha".
axiom2texUCa('*2'a) --> " ", bs, "star_2 ", bs, "alpha".
axiom2texUCb(-b)    --> " ", bs,"lnot ", bs, "beta".
axiom2texUCb('o'b)  --> " ", bs, "circ ", bs, "beta".
axiom2texUCb('*1'b) --> " ", bs, "star_1 ", bs, "beta".
axiom2texUCb('*2'b) --> " ", bs, "star_2 ", bs, "beta".
axiom2texBC(X & Y)  --> axiom2texC(X), " ", bs, "land ", axiom2texC(Y).
axiom2texBC(X v Y)  --> axiom2texC(X), " ", bs, "lor ", axiom2texC(Y).
axiom2texBC(X -> Y)  --> axiom2texC(X), " ", bs, "ra ", axiom2texC(Y).


%% test if a expression is the one that appears in the conclusion

isLogical(-X) 	  --> isLogicalF(X).
isLogical('o'X)	  --> isLogicalF(X).
isLogical('*1'X)  --> isLogicalF(X).
isLogical('*2'X)  --> isLogicalF(X).
isLogicalF(-X) 	  --> axiom2texC(X).
isLogicalF('o'X)  --> axiom2texC(X).
isLogicalF('*1'X)  --> axiom2texC(X).
isLogicalF('*2'X)  --> axiom2texC(X).
isLogicalF(X & Y) --> axiom2texC(X), axiom2texC(Y).
isLogicalF(X v Y) --> axiom2texC(X), axiom2texC(Y).
isLogicalF(X -> Y)--> axiom2texC(X), axiom2texC(Y).

%isLogical1(-X) 	  --> axiom2texC(X).
%isLogical1('o'X)  --> axiom2texC(X).
%isLogical1('*1'X) --> axiom2texC(X).
%isLogical1('*2'X) --> axiom2texC(X).
%isLogical1(-X) 	  --> isLogical1(X).
%isLogical1('o'X)  --> axiom2texC(X).
%isLogical1('*1'X) --> isLogical1(X).
%isLogical1('*2'X) --> isLogical1(X).


print_phrase(V) :-
   phrase(V, S),
   name(A, S),
   write(A).



