%%% Filename : p_grammar.pl
%%% Date     : 03.04.2012, extended: 11.11.2013
%%% Contents : DCG grammar for tex-output and formula-check
%%% Author   : Lara Katharina Spendier <lara@logic.at>

bs        	  --> [92].
eol 	  	  --> [13,10].

%% Definitions for the style-file

texNewAxiom(C)		--> bs, "def", bs, "NewAxiom", "{$", C, "$}", eol.
texNewAxiomNotMath(C)	--> bs, "def", bs, "NewAxiomNotMath", "{", C, "}", eol.
texNewRuleStart   	--> bs, "def", bs, "NewRule", "{".
texNewRuleEnd	  	--> "}", eol.
texAxiomClass(C)  	--> bs, "def", bs, "AxiomClass", "{$", bs, "mathcal", C, "$}", eol.
texBaseCalculus(C) 	--> bs, "def", bs, "BaseCalculus{",bs,C,"}",eol.
texEquivalences(C) 	--> bs, "def", bs, "Equivalences{",bs,C,"}",eol.
texWrittenBC(C)	   	--> bs, "def", bs, "WrittenBC{",C,"}",eol.
texRule(C)		--> bs, "def", bs, "WRule{",C,"}",eol.
texAxiomNoun(C)		--> bs, "def", bs, "AxiomNoun{",C,"}",eol.
texAxiomVerb(C)		--> bs, "def", bs, "AxiomVerb{",C,"}",eol.
texVM(C)		--> bs, "def", bs, "VMNotMath{",bs,"{",C,bs,"}","}",eol.
texDM(C)		--> bs, "def", bs, "DMNotMath{",bs,"{",C,bs,"}","}",eol.
texStar1TT(C)		--> bs, "def", bs, "StarOneTT{",bs,"begin{array}{c||c}",bs,"star_1 & ",bs,bs,bs,"hline",bs,"hline",C,bs,"end{array}}",eol.
texStar2TT(C)		--> bs, "def", bs, "StarTwoTT{",bs,"begin{array}{c||c}",bs,"star_2 & ",bs,bs,bs,"hline",bs,"hline",C,bs,"end{array}}",eol.
texAndTT(C)		--> bs, "def", bs, "AndTT{", bs, "begin{array}{c|",C,bs,"end{array}}",eol.
texOrTT(C)		--> bs, "def", bs, "OrTT{", bs, "begin{array}{c|",C,bs,"end{array}}",eol.
texImpliesTT(C)		--> bs, "def", bs, "ImpliesTT{", bs, "begin{array}{c|",C,bs,"end{array}}",eol.
texBaseCalculusTab(C) 	--> bs, "def", bs, "BaseCalculusTab{",bs,C,"}",eol.
texBaseCalculusDesc(C)	--> bs, "def", bs, "BaseCalculusDesc{", bs, C, "}", eol.
texNameBaseLogic(C)	--> bs, "def", bs, "NameBaseLogic{",C,"}",eol.
texNameBaseLogicExpl(C)	--> bs, "def", bs, "NameBaseLogicExpl{",C,"}",eol.
texDescBaseLogic(C)	--> bs, "def", bs, "DescBaseLogic{",bs,C,"}",eol.
texNameBaseCalculus(C)	--> bs, "def", bs, "NameBaseCalculus{",C,"}",eol.
texNameBaseCalculusP(C)	--> bs, "def", bs, "NameBaseCalculusP{",C,"}",eol.
texConnectiveNoun	--> bs, "def", bs, "ConnectiveNoun{connective}",eol.
texConnectiveNouns	--> bs, "def", bs, "ConnectiveNoun{connectives}",eol.
texUnaryConnective	--> bs, "def", bs, "UnaryConnective{$",bs,"star_1$}",eol.
texUnaryConnectives	--> bs, "def", bs, "UnaryConnective{$",bs,"star_1,",bs,"star_2$}",eol.
texAnalyticity(C)	--> bs, "def", bs, "ThmAnalyticity{", bs, C, "}",eol.
texArticleAnalytic(C)	--> bs, "def", bs, "ArticleAnalytic{", C, "}",eol.
texIntro(C)		--> bs, "def", bs, "Intro{", bs, C, "}",eol.
texTitle(C)		--> bs, "def", bs, "AnalyticTitle{", C, "}",eol.
texTruthTables(C)	--> bs, "def", bs, "TruthTables{",bs,C,"}",eol.

%% tex-rewriting of the (hyper)sequent rules with >= 0 premises

texNewAxC(C)      --> bs, "AxiomC{$",C,"$}",eol.
texNewUnC(C)	  --> bs, "UnaryInfC{$",C,"$}",eol.
texNoLine	  --> bs, "noLine", eol.

texNewUnCon(C)    --> bs, "UnaryInfC{$",C,"$}",bs,"DisplayProof",bs,"hspace*{0.3cm}",bs,"vspace*{0.3cm}",eol.
texNewBinCon(C)   --> bs, "BinaryInfC{$",C,"$}",bs,"DisplayProof",bs,"hspace*{0.3cm}",bs,"vspace*{0.3cm}",eol.
texNewTriCon(C)   --> bs, "TrinaryInfC{$",C,"$}",bs,"DisplayProof",bs,"hspace*{0.3cm}",bs,"vspace*{0.3cm}",eol.

rule2tex([[H|T],[H1|T1]])  --> rule2tex([H|T]),bs,"Ra",rule2tex([H1|T1]).
rule2tex('G')     --> bs,"Gamma".
rule2tex('D')     --> bs,"Delta".
rule2tex('')      --> "".
rule2tex([])	  --> "".
rule2tex(X)	  --> axiom2texC(X).
rule2tex(X)	  --> axiom2tex(X).
rule2tex([H|[]])  --> rule2tex(H).
rule2tex([H|T])   --> rule2tex(H), ",", rule2tex(T).

nr2tex(N)	  --> {number_codes(N, Cs)}, Cs.

%% axiom2texS  ... start of grammar S
%% axiom2texRP ... subformula that contains the logical formula of the form: o x
%% axiom2texR1 ... subformula that contains the logical formula of the form: o o x
%% axiom2texR2 ... subformula that contains the logical formula of the form: o (x v y)
%% axiom2texP1 ... subformula that contains no nested unary connectives, only o x
%% axiom2texP2 ... subformula that contains no nested unary connectives, only o x, o y
%% axiom2texC ... subformula that is an atom
%% axiom2texUC ... subformula that is an atom bounded by one unary connective
%% axiom2texBC ... subformula that is two atoms bounded by one binary connective
%% ...a, ....b ... means that the propositional variable is either a or b (we have to keep track due to the restrictions of our grammar)
%% _A, _O, _I  ... keep track of the bracketing (And, Or, Implication)


axioms2tex(X ';' Y) --> axioms2tex(X), axioms2tex(Y).
axioms2tex(X)	  --> axiom2tex(X).
axiom2tex(X)	  --> axiom2texS(X).

%% S = RP | R1 | R2
%% 
%axiom2texS(-X) 	   --> " ", bs, "lnot ", axiom2texC(X).
%axiom2texS(-X) 	   --> " ", bs, "lnot ", axiom2texUC(X).
%axiom2texS(-X) 	   --> " ", bs, "lnot (", axiom2texBC(X), ")".
%axiom2texS('o'X)   --> " ", bs, "circ ", axiom2texC(X).
%axiom2texS('o'X)   --> " ", bs, "circ ", axiom2texUC(X).
%axiom2texS('o'X)   --> " ", bs, "circ (", axiom2texBC(X), ")".
axiom2texS('*1'X)  --> " ", bs, "star_1 ", axiom2texC(X).
axiom2texS('*1'X)  --> " ", bs, "star_1 ", axiom2texUC(X).
axiom2texS('*1'X)  --> " ", bs, "star_1 (", axiom2texBC(X), ")".
axiom2texS('*2'X)  --> " ", bs, "star_2 ", axiom2texC(X).
axiom2texS('*2'X)  --> " ", bs, "star_2 ", axiom2texUC(X).
axiom2texS('*2'X)  --> " ", bs, "star_2 (", axiom2texBC(X), ")".
axiom2texS(X & Y)  --> axiom2texRPaA(X), " ", bs, "land ", axiom2texP1aA(Y).
axiom2texS(X & Y)  --> axiom2texP1aA(X), " ", bs, "land ", axiom2texRPaA(Y).
axiom2texS(X & Y)  --> axiom2texRPbA(X), " ", bs, "land ", axiom2texP1bA(Y).
axiom2texS(X & Y)  --> axiom2texP1bA(X), " ", bs, "land ", axiom2texRPbA(Y).
axiom2texS(X & Y)  --> axiom2texR1aA(X), " ", bs, "land ", axiom2texP1aA(Y).
axiom2texS(X & Y)  --> axiom2texP1aA(X), " ", bs, "land ", axiom2texR1aA(Y).
axiom2texS(X & Y)  --> axiom2texR1bA(X), " ", bs, "land ", axiom2texP1bA(Y).
axiom2texS(X & Y)  --> axiom2texP1bA(X), " ", bs, "land ", axiom2texR1bA(Y).
axiom2texS(X & Y)  --> axiom2texR2A(X), " ", bs, "land ", axiom2texP2A(Y).
axiom2texS(X & Y)  --> axiom2texP2A(X), " ", bs, "land ", axiom2texR2A(Y).
axiom2texS(X v Y)  --> axiom2texRPaO(X), " ", bs, "lor ", axiom2texP1aO(Y).
axiom2texS(X v Y)  --> axiom2texP1aO(X), " ", bs, "lor ", axiom2texRPaO(Y).
axiom2texS(X v Y)  --> axiom2texRPbO(X), " ", bs, "lor ", axiom2texP1bO(Y).
axiom2texS(X v Y)  --> axiom2texP1bO(X), " ", bs, "lor ", axiom2texRPbO(Y).
axiom2texS(X v Y)  --> axiom2texR1aO(X), " ", bs, "lor ", axiom2texP1aO(Y).
axiom2texS(X v Y)  --> axiom2texP1aO(X), " ", bs, "lor ", axiom2texR1aO(Y).
axiom2texS(X v Y)  --> axiom2texR1bO(X), " ", bs, "lor ", axiom2texP1bO(Y).
axiom2texS(X v Y)  --> axiom2texP1bO(X), " ", bs, "lor ", axiom2texR1bO(Y).
axiom2texS(X v Y)  --> axiom2texR2O(X), " ", bs, "lor ", axiom2texP2O(Y).
axiom2texS(X v Y)  --> axiom2texP2O(X), " ", bs, "lor ", axiom2texR2O(Y).
axiom2texS(X -> Y) --> axiom2texRPaI(X), " ", bs, "ra ", axiom2texP1aI(Y).
axiom2texS(X -> Y) --> axiom2texP1aI(X), " ", bs, "ra ", axiom2texRPaI(Y).
axiom2texS(X -> Y) --> axiom2texRPbI(X), " ", bs, "ra ", axiom2texP1bI(Y).
axiom2texS(X -> Y) --> axiom2texP1bI(X), " ", bs, "ra ", axiom2texRPbI(Y).
axiom2texS(X -> Y) --> axiom2texR1aI(X), " ", bs, "ra ", axiom2texP1aI(Y).
axiom2texS(X -> Y) --> axiom2texP1aI(X), " ", bs, "ra ", axiom2texR1aI(Y).
axiom2texS(X -> Y) --> axiom2texR1bI(X), " ", bs, "ra ", axiom2texP1bI(Y).
axiom2texS(X -> Y) --> axiom2texP1bI(X), " ", bs, "ra ", axiom2texR1bI(Y).
axiom2texS(X -> Y) --> axiom2texR2I(X), " ", bs, "ra ", axiom2texP2I(Y).
axiom2texS(X -> Y) --> axiom2texP2I(X), " ", bs, "ra ", axiom2texR2I(Y).

%% RP - divided into And, Or, Implication (for the bracketing)
%axiom2texRPaA(-X)     --> " ", bs, "lnot ", axiom2texCa(X).
%axiom2texRPaA('o'X)   --> " ", bs, "circ ", axiom2texCa(X).
axiom2texRPaA('*1'X)  --> " ", bs, "star_1 ", axiom2texCa(X).
axiom2texRPaA('*2'X)  --> " ", bs, "star_2 ", axiom2texCa(X).
axiom2texRPaA(X & Y)  --> axiom2texRPaA(X), " ", bs, "land ", axiom2texP1aA(Y).
axiom2texRPaA(X & Y)  --> axiom2texP1aA(X), " ", bs, "land ", axiom2texRPaA(Y).
axiom2texRPaA(X v Y)  --> "(", axiom2texRPaO(X), " ", bs, "lor ", axiom2texP1aO(Y), ")".
axiom2texRPaA(X v Y)  --> "(", axiom2texP1aO(X), " ", bs, "lor ", axiom2texRPaO(Y), ")".
axiom2texRPaA(X -> Y) --> "(", axiom2texRPaI(X), " ", bs, "ra ", axiom2texP1aI(Y), ")".
axiom2texRPaA(X -> Y) --> "(", axiom2texP1aI(X), " ", bs, "ra ", axiom2texRPaI(Y), ")".
%axiom2texRPbA(-X)     --> " ", bs, "lnot ", axiom2texCb(X).
%axiom2texRPbA('o'X)   --> " ", bs, "circ ", axiom2texCb(X).
axiom2texRPbA('*1'X)  --> " ", bs, "star_1 ", axiom2texCb(X).
axiom2texRPbA('*2'X)  --> " ", bs, "star_2 ", axiom2texCb(X).
axiom2texRPbA(X & Y)  --> axiom2texRPbA(X), " ", bs, "land ", axiom2texP1bA(Y).
axiom2texRPbA(X & Y)  --> axiom2texP1bA(X), " ", bs, "land ", axiom2texRPbA(Y).
axiom2texRPbA(X v Y)  --> "(", axiom2texRPbO(X), " ", bs, "lor ", axiom2texP1bO(Y), ")".
axiom2texRPbA(X v Y)  --> "(", axiom2texP1bO(X), " ", bs, "lor ", axiom2texRPbO(Y), ")".
axiom2texRPbA(X -> Y) --> "(", axiom2texRPbI(X), " ", bs, "ra ", axiom2texP1bI(Y), ")".
axiom2texRPbA(X -> Y) --> "(", axiom2texP1bI(X), " ", bs, "ra ", axiom2texRPbI(Y), ")".

%axiom2texRPaO(-X)     --> " ", bs, "lnot ", axiom2texCa(X).
%axiom2texRPaO('o'X)   --> " ", bs, "circ ", axiom2texCa(X).
axiom2texRPaO('*1'X)  --> " ", bs, "star_1 ", axiom2texCa(X).
axiom2texRPaO('*2'X)  --> " ", bs, "star_2 ", axiom2texCa(X).
axiom2texRPaO(X & Y)  --> "(", axiom2texRPaA(X), " ", bs, "land ", axiom2texP1aA(Y), ")".
axiom2texRPaO(X & Y)  --> "(", axiom2texP1aA(X), " ", bs, "land ", axiom2texRPaA(Y), ")".
axiom2texRPaO(X v Y)  --> axiom2texRPaO(X), " ", bs, "lor ", axiom2texP1aO(Y).
axiom2texRPaO(X v Y)  --> axiom2texP1aO(X), " ", bs, "lor ", axiom2texRPaO(Y).
axiom2texRPaO(X -> Y) --> "(", axiom2texRPaI(X), " ", bs, "ra ", axiom2texP1aI(Y), ")".
axiom2texRPaO(X -> Y) --> "(", axiom2texP1aI(X), " ", bs, "ra ", axiom2texRPaI(Y), ")".
%axiom2texRPbO(-X)     --> " ", bs, "lnot ", axiom2texCb(X).
%axiom2texRPbO('o'X)   --> " ", bs, "circ ", axiom2texCb(X).
axiom2texRPbO('*1'X)  --> " ", bs, "star_1 ", axiom2texCb(X).
axiom2texRPbO('*2'X)  --> " ", bs, "star_2 ", axiom2texCb(X).
axiom2texRPbO(X & Y)  --> "(", axiom2texRPbA(X), " ", bs, "land ", axiom2texP1bA(Y), ")".
axiom2texRPbO(X & Y)  --> "(", axiom2texP1bA(X), " ", bs, "land ", axiom2texRPbA(Y), ")".
axiom2texRPbO(X v Y)  --> axiom2texRPbO(X), " ", bs, "lor ", axiom2texP1bO(Y).
axiom2texRPbO(X v Y)  --> axiom2texP1bO(X), " ", bs, "lor ", axiom2texRPbO(Y).
axiom2texRPbO(X -> Y) --> "(", axiom2texRPbI(X), " ", bs, "ra ", axiom2texP1bI(Y), ")".
axiom2texRPbO(X -> Y) --> "(", axiom2texP1bI(X), " ", bs, "ra ", axiom2texRPbI(Y), ")".

%axiom2texRPaI(-X)     --> " ", bs, "lnot ", axiom2texCa(X).
%axiom2texRPaI('o'X)   --> " ", bs, "circ ", axiom2texCa(X).
axiom2texRPaI('*1'X)  --> " ", bs, "star_1 ", axiom2texCa(X).
axiom2texRPaI('*2'X)  --> " ", bs, "star_2 ", axiom2texCa(X).
axiom2texRPaI(X & Y)  --> "(", axiom2texRPaA(X), " ", bs, "land ", axiom2texP1aA(Y), ")".
axiom2texRPaI(X & Y)  --> "(", axiom2texP1aA(X), " ", bs, "land ", axiom2texRPaA(Y), ")".
axiom2texRPaI(X v Y)  --> "(", axiom2texRPaO(X), " ", bs, "lor ", axiom2texP1aO(Y), ")".
axiom2texRPaI(X v Y)  --> "(", axiom2texP1aO(X), " ", bs, "lor ", axiom2texRPaO(Y), ")".
axiom2texRPaI(X -> Y) --> "(", axiom2texRPaI(X), " ", bs, "ra ", axiom2texP1aI(Y), ")".
axiom2texRPaI(X -> Y) --> "(", axiom2texP1aI(X), " ", bs, "ra ", axiom2texRPaI(Y), ")".
%axiom2texRPbI(-X)     --> " ", bs, "lnot ", axiom2texCb(X).
%axiom2texRPbI('o'X)   --> " ", bs, "circ ", axiom2texCb(X).
axiom2texRPbI('*1'X)  --> " ", bs, "star_1 ", axiom2texCb(X).
axiom2texRPbI('*2'X)  --> " ", bs, "star_2 ", axiom2texCb(X).
axiom2texRPbI(X & Y)  --> "(", axiom2texRPbA(X), " ", bs, "land ", axiom2texP1bA(Y), ")".
axiom2texRPbI(X & Y)  --> "(", axiom2texP1bA(X), " ", bs, "land ", axiom2texRPbA(Y), ")".
axiom2texRPbI(X v Y)  --> "(", axiom2texRPbO(X), " ", bs, "lor ", axiom2texP1bO(Y), ")".
axiom2texRPbI(X v Y)  --> "(", axiom2texP1bO(X), " ", bs, "lor ", axiom2texRPbO(Y), ")".
axiom2texRPbI(X -> Y) --> "(", axiom2texRPbI(X), " ", bs, "ra ", axiom2texP1bI(Y), ")".
axiom2texRPbI(X -> Y) --> "(", axiom2texP1bI(X), " ", bs, "ra ", axiom2texRPbI(Y), ")".

%% R1 - divided into And, Or, Implication (for the bracketing)

%axiom2texR1aA(-X)     --> " ", bs, "lnot ", axiom2texUCa(X).
%axiom2texR1aA('o'X)   --> " ", bs, "circ ", axiom2texUCa(X).
axiom2texR1aA('*1'X)  --> " ", bs, "star_1 ", axiom2texUCa(X).
axiom2texR1aA('*2'X)  --> " ", bs, "star_2 ", axiom2texUCa(X).
axiom2texR1aA(X & Y)  --> axiom2texR1aA(X), " ", bs, "land ", axiom2texP1aA(Y).
axiom2texR1aA(X & Y)  --> axiom2texP1aA(X), " ", bs, "land ", axiom2texR1aA(Y).
axiom2texR1aA(X v Y)  --> "(", axiom2texR1aO(X), " ", bs, "lor ", axiom2texP1aO(Y), ")".
axiom2texR1aA(X v Y)  --> "(", axiom2texP1aO(X), " ", bs, "lor ", axiom2texR1aO(Y), ")".
axiom2texR1aA(X -> Y) --> "(", axiom2texR1aI(X), " ", bs, "ra ", axiom2texP1aI(Y), ")".
axiom2texR1aA(X -> Y) --> "(", axiom2texP1aI(X), " ", bs, "ra ", axiom2texR1aI(Y), ")".
%axiom2texR1bA(-X)     --> " ", bs, "lnot ", axiom2texUCb(X).
%axiom2texR1bA('o'X)   --> " ", bs, "circ ", axiom2texUCb(X).
axiom2texR1bA('*1'X)  --> " ", bs, "star_1 ", axiom2texUCb(X).
axiom2texR1bA('*2'X)  --> " ", bs, "star_2 ", axiom2texUCb(X).
axiom2texR1bA(X & Y)  --> axiom2texR1bA(X), " ", bs, "land ", axiom2texP1bA(Y).
axiom2texR1bA(X & Y)  --> axiom2texP1bA(X), " ", bs, "land ", axiom2texR1bA(Y).
axiom2texR1bA(X v Y)  --> "(", axiom2texR1bO(X), " ", bs, "lor ", axiom2texP1bO(Y), ")".
axiom2texR1bA(X v Y)  --> "(", axiom2texP1bO(X), " ", bs, "lor ", axiom2texR1bO(Y), ")".
axiom2texR1bA(X -> Y) --> "(", axiom2texR1bI(X), " ", bs, "ra ", axiom2texP1bI(Y), ")".
axiom2texR1bA(X -> Y) --> "(", axiom2texP1bI(X), " ", bs, "ra ", axiom2texR1bI(Y), ")".

%axiom2texR1aO(-X)     --> " ", bs, "lnot ", axiom2texUCa(X).
%axiom2texR1aO('o'X)   --> " ", bs, "circ ", axiom2texUCa(X).
axiom2texR1aO('*1'X)  --> " ", bs, "star_1 ", axiom2texUCa(X).
axiom2texR1aO('*2'X)  --> " ", bs, "star_2 ", axiom2texUCa(X).
axiom2texR1aO(X & Y)  --> "(", axiom2texR1aA(X), " ", bs, "land ", axiom2texP1aA(Y), ")".
axiom2texR1aO(X & Y)  --> "(", axiom2texP1aA(X), " ", bs, "land ", axiom2texR1aA(Y), ")".
axiom2texR1aO(X v Y)  --> axiom2texR1aO(X), " ", bs, "lor ", axiom2texP1aO(Y).
axiom2texR1aO(X v Y)  --> axiom2texP1aO(X), " ", bs, "lor ", axiom2texR1aO(Y).
axiom2texR1aO(X -> Y) --> "(", axiom2texR1aI(X), " ", bs, "ra ", axiom2texP1aI(Y), ")".
axiom2texR1aO(X -> Y) --> "(", axiom2texP1aI(X), " ", bs, "ra ", axiom2texR1aI(Y), ")".
%axiom2texR1bO(-X)     --> " ", bs, "lnot ", axiom2texUCb(X).
%axiom2texR1bO('o'X)   --> " ", bs, "circ ", axiom2texUCb(X).
axiom2texR1bO('*1'X)  --> " ", bs, "star_1 ", axiom2texUCb(X).
axiom2texR1bO('*2'X)  --> " ", bs, "star_2 ", axiom2texUCb(X).
axiom2texR1bO(X & Y)  --> "(", axiom2texR1bA(X), " ", bs, "land ", axiom2texP1bA(Y), ")".
axiom2texR1bO(X & Y)  --> "(", axiom2texP1bA(X), " ", bs, "land ", axiom2texR1bA(Y), ")".
axiom2texR1bO(X v Y)  --> axiom2texR1bO(X), " ", bs, "lor ", axiom2texP1bO(Y).
axiom2texR1bO(X v Y)  --> axiom2texP1bO(X), " ", bs, "lor ", axiom2texR1bO(Y).
axiom2texR1bO(X -> Y) --> "(", axiom2texR1bI(X), " ", bs, "ra ", axiom2texP1bI(Y), ")".
axiom2texR1bO(X -> Y) --> "(", axiom2texP1bI(X), " ", bs, "ra ", axiom2texR1bI(Y), ")".

%axiom2texR1aI(-X)     --> " ", bs, "lnot ", axiom2texUCa(X).
%axiom2texR1aI('o'X)   --> " ", bs, "circ ", axiom2texUCa(X).
axiom2texR1aI('*1'X)  --> " ", bs, "star_1 ", axiom2texUCa(X).
axiom2texR1aI('*2'X)  --> " ", bs, "star_2 ", axiom2texUCa(X).
axiom2texR1aI(X & Y)  --> "(", axiom2texR1aA(X), " ", bs, "land ", axiom2texP1aA(Y), ")".
axiom2texR1aI(X & Y)  --> "(", axiom2texP1aA(X), " ", bs, "land ", axiom2texR1aA(Y), ")".
axiom2texR1aI(X v Y)  --> "(", axiom2texR1aO(X), " ", bs, "lor ", axiom2texP1aO(Y), ")".
axiom2texR1aI(X v Y)  --> "(", axiom2texP1aO(X), " ", bs, "lor ", axiom2texR1aO(Y), ")".
axiom2texR1aI(X -> Y) --> "(", axiom2texR1aI(X), " ", bs, "ra ", axiom2texP1aI(Y), ")".
axiom2texR1aI(X -> Y) --> "(", axiom2texP1aI(X), " ", bs, "ra ", axiom2texR1aI(Y), ")".
%axiom2texR1bI(-X)     --> " ", bs, "lnot ", axiom2texUCb(X).
%axiom2texR1bI('o'X)   --> " ", bs, "circ ", axiom2texUCb(X).
axiom2texR1bI('*1'X)  --> " ", bs, "star_1 ", axiom2texUCb(X).
axiom2texR1bI('*2'X)  --> " ", bs, "star_2 ", axiom2texUCb(X).
axiom2texR1bI(X & Y)  --> "(", axiom2texR1bA(X), " ", bs, "land ", axiom2texP1bA(Y), ")".
axiom2texR1bI(X & Y)  --> "(", axiom2texP1bA(X), " ", bs, "land ", axiom2texR1bA(Y), ")".
axiom2texR1bI(X v Y)  --> "(", axiom2texR1bO(X), " ", bs, "lor ", axiom2texP1bO(Y), ")".
axiom2texR1bI(X v Y)  --> "(", axiom2texP1bO(X), " ", bs, "lor ", axiom2texR1bO(Y), ")".
axiom2texR1bI(X -> Y) --> "(", axiom2texR1bI(X), " ", bs, "ra ", axiom2texP1bI(Y), ")".
axiom2texR1bI(X -> Y) --> "(", axiom2texP1bI(X), " ", bs, "ra ", axiom2texR1bI(Y), ")".

%% P1 - divided into And, Or, Implication (for the bracketing)

axiom2texP1aA(X)      --> axiom2texC(X).
%axiom2texP1aA(-X)     --> " ", bs, "lnot ", axiom2texCa(X).
%axiom2texP1aA('o'X)   --> " ", bs, "circ ", axiom2texCa(X).
axiom2texP1aA('*1'X)  --> " ", bs, "star_1 ", axiom2texCa(X).
axiom2texP1aA('*2'X)  --> " ", bs, "star_2 ", axiom2texCa(X).
axiom2texP1aA(X & Y)  --> axiom2texP1aA(X), " ", bs, "land ", axiom2texP1aA(Y).
axiom2texP1aA(X & Y)  --> axiom2texP1aA(X), " ", bs, "land ", axiom2texP1aA(Y).
axiom2texP1aA(X v Y)  --> "(", axiom2texP1aO(X), " ", bs, "lor ", axiom2texP1aO(Y), ")".
axiom2texP1aA(X v Y)  --> "(", axiom2texP1aO(X), " ", bs, "lor ", axiom2texP1aO(Y), ")".
axiom2texP1aA(X -> Y) --> "(", axiom2texP1aI(X), " ", bs, "ra ", axiom2texP1aI(Y), ")".
axiom2texP1aA(X -> Y) --> "(", axiom2texP1aI(X), " ", bs, "ra ", axiom2texP1aI(Y), ")".
axiom2texP1bA(X)      --> axiom2texC(X).
%axiom2texP1bA(-X)     --> " ", bs, "lnot ", axiom2texCb(X).
%axiom2texP1bA('o'X)   --> " ", bs, "circ ", axiom2texCb(X).
axiom2texP1bA('*1'X)  --> " ", bs, "star_1 ", axiom2texCb(X).
axiom2texP1bA('*2'X)  --> " ", bs, "star_2 ", axiom2texCb(X).
axiom2texP1bA(X & Y)  --> axiom2texP1bA(X), " ", bs, "land ", axiom2texP1bA(Y).
axiom2texP1bA(X & Y)  --> axiom2texP1bA(X), " ", bs, "land ", axiom2texP1bA(Y).
axiom2texP1bA(X v Y)  --> "(", axiom2texP1bO(X), " ", bs, "lor ", axiom2texP1bO(Y), ")".
axiom2texP1bA(X v Y)  --> "(", axiom2texP1bO(X), " ", bs, "lor ", axiom2texP1bO(Y), ")".
axiom2texP1bA(X -> Y) --> "(", axiom2texP1bI(X), " ", bs, "ra ", axiom2texP1bI(Y), ")".
axiom2texP1bA(X -> Y) --> "(", axiom2texP1bI(X), " ", bs, "ra ", axiom2texP1bI(Y), ")".

axiom2texP1aO(X)      --> axiom2texC(X).
%axiom2texP1aO(-X)     --> " ", bs, "lnot ", axiom2texCa(X).
%axiom2texP1aO('o'X)   --> " ", bs, "circ ", axiom2texCa(X).
axiom2texP1aO('*1'X)  --> " ", bs, "star_1 ", axiom2texCa(X).
axiom2texP1aO('*2'X)  --> " ", bs, "star_2 ", axiom2texCa(X).
axiom2texP1aO(X & Y)  --> "(", axiom2texP1aA(X), " ", bs, "land ", axiom2texP1aA(Y), ")".
axiom2texP1aO(X & Y)  --> "(", axiom2texP1aA(X), " ", bs, "land ", axiom2texP1aA(Y), ")".
axiom2texP1aO(X v Y)  --> axiom2texP1aO(X), " ", bs, "lor ", axiom2texP1aO(Y).
axiom2texP1aO(X v Y)  --> axiom2texP1aO(X), " ", bs, "lor ", axiom2texP1aO(Y).
axiom2texP1aO(X -> Y) --> "(", axiom2texP1aI(X), " ", bs, "ra ", axiom2texP1aI(Y), ")".
axiom2texP1aO(X -> Y) --> "(", axiom2texP1aI(X), " ", bs, "ra ", axiom2texP1aI(Y), ")".
axiom2texP1bO(X)      --> axiom2texC(X).
%axiom2texP1bO(-X)     --> " ", bs, "lnot ", axiom2texCb(X).
%axiom2texP1bO('o'X)   --> " ", bs, "circ ", axiom2texCb(X).
axiom2texP1bO('*1'X)  --> " ", bs, "star_1 ", axiom2texCb(X).
axiom2texP1bO('*2'X)  --> " ", bs, "star_2 ", axiom2texCb(X).
axiom2texP1bO(X & Y)  --> "(", axiom2texP1bA(X), " ", bs, "land ", axiom2texP1bA(Y), ")".
axiom2texP1bO(X & Y)  --> "(", axiom2texP1bA(X), " ", bs, "land ", axiom2texP1bA(Y), ")".
axiom2texP1bO(X v Y)  --> axiom2texP1bO(X), " ", bs, "lor ", axiom2texP1bO(Y).
axiom2texP1bO(X v Y)  --> axiom2texP1bO(X), " ", bs, "lor ", axiom2texP1bO(Y).
axiom2texP1bO(X -> Y) --> "(", axiom2texP1bI(X), " ", bs, "ra ", axiom2texP1bI(Y), ")".
axiom2texP1bO(X -> Y) --> "(", axiom2texP1bI(X), " ", bs, "ra ", axiom2texP1bI(Y), ")".

axiom2texP1aI(X)      --> axiom2texC(X).
%axiom2texP1aI(-X)     --> " ", bs, "lnot ", axiom2texCa(X).
%axiom2texP1aI('o'X)   --> " ", bs, "circ ", axiom2texCa(X).
axiom2texP1aI('*1'X)  --> " ", bs, "star_1 ", axiom2texCa(X).
axiom2texP1aI('*2'X)  --> " ", bs, "star_2 ", axiom2texCa(X).
axiom2texP1aI(X & Y)  --> "(", axiom2texP1aA(X), " ", bs, "land ", axiom2texP1aA(Y), ")".
axiom2texP1aI(X & Y)  --> "(", axiom2texP1aA(X), " ", bs, "land ", axiom2texP1aA(Y), ")".
axiom2texP1aI(X v Y)  --> "(", axiom2texP1aO(X), " ", bs, "lor ", axiom2texP1aO(Y), ")".
axiom2texP1aI(X v Y)  --> "(", axiom2texP1aO(X), " ", bs, "lor ", axiom2texP1aO(Y), ")".
axiom2texP1aI(X -> Y) --> "(", axiom2texP1aI(X), " ", bs, "ra ", axiom2texP1aI(Y), ")".
axiom2texP1aI(X -> Y) --> "(", axiom2texP1aI(X), " ", bs, "ra ", axiom2texP1aI(Y), ")".
axiom2texP1bI(X)      --> axiom2texC(X).
%axiom2texP1bI(-X)     --> " ", bs, "lnot ", axiom2texCb(X).
%axiom2texP1bI('o'X)   --> " ", bs, "circ ", axiom2texCb(X).
axiom2texP1bI('*1'X)  --> " ", bs, "star_1 ", axiom2texCb(X).
axiom2texP1bI('*2'X)  --> " ", bs, "star_2 ", axiom2texCb(X).
axiom2texP1bI(X & Y)  --> "(", axiom2texP1bA(X), " ", bs, "land ", axiom2texP1bA(Y), ")".
axiom2texP1bI(X & Y)  --> "(", axiom2texP1bA(X), " ", bs, "land ", axiom2texP1bA(Y), ")".
axiom2texP1bI(X v Y)  --> "(", axiom2texP1bO(X), " ", bs, "lor ", axiom2texP1bO(Y), ")".
axiom2texP1bI(X v Y)  --> "(", axiom2texP1bO(X), " ", bs, "lor ", axiom2texP1bO(Y), ")".
axiom2texP1bI(X -> Y) --> "(", axiom2texP1bI(X), " ", bs, "ra ", axiom2texP1bI(Y), ")".
axiom2texP1bI(X -> Y) --> "(", axiom2texP1bI(X), " ", bs, "ra ", axiom2texP1bI(Y), ")".

%% R2 - divided into And, Or, Implication (for the bracketing)

%axiom2texR2A(-X)     --> " ", bs, "lnot (", axiom2texBC(X), ")".
%axiom2texR2A('o'X)   --> " ", bs, "circ (", axiom2texBC(X), ")".
axiom2texR2A('*1'X)  --> " ", bs, "star_1 (", axiom2texBC(X), ")".
axiom2texR2A('*2'X)  --> " ", bs, "star_2 (", axiom2texBC(X), ")".
axiom2texR2A(X & Y)  --> axiom2texR2A(X), " ", bs, "land ", axiom2texP2A(Y).
axiom2texR2A(X & Y)  --> axiom2texP2A(X), " ", bs, "land ", axiom2texR2A(Y).
axiom2texR2A(X v Y)  --> "(", axiom2texR2O(X), " ", bs, "lor ", axiom2texP2O(Y), ")".
axiom2texR2A(X v Y)  --> "(", axiom2texP2O(X), " ", bs, "lor ", axiom2texR2O(Y), ")".
axiom2texR2A(X -> Y)  --> "(", axiom2texR2I(X), " ", bs, "ra ", axiom2texP2I(Y), ")".
axiom2texR2A(X -> Y)  --> "(", axiom2texP2I(X), " ", bs, "ra ", axiom2texR2I(Y), ")".

%axiom2texR2O(-X)     --> " ", bs, "lnot (", axiom2texBC(X), ")".
%axiom2texR2O('o'X)   --> " ", bs, "circ (", axiom2texBC(X), ")".
axiom2texR2O('*1'X)  --> " ", bs, "star_1 (", axiom2texBC(X), ")".
axiom2texR2O('*2'X)  --> " ", bs, "star_2 (", axiom2texBC(X), ")".
axiom2texR2O(X & Y)  --> "(", axiom2texR2A(X), " ", bs, "land ", axiom2texP2A(Y), ")".
axiom2texR2O(X & Y)  --> "(", axiom2texP2A(X), " ", bs, "land ", axiom2texR2A(Y), ")".
axiom2texR2O(X v Y)  --> axiom2texR2O(X), " ", bs, "lor ", axiom2texP2O(Y).
axiom2texR2O(X v Y)  --> axiom2texP2O(X), " ", bs, "lor ", axiom2texR2O(Y).
axiom2texR2O(X -> Y) --> "(", axiom2texR2I(X), " ", bs, "ra ", axiom2texP2I(Y), ")".
axiom2texR2O(X -> Y) --> "(", axiom2texP2I(X), " ", bs, "ra ", axiom2texR2I(Y), ")".

%axiom2texR2I(-X)     --> " ", bs, "lnot (", axiom2texBC(X), ")".
%axiom2texR2I('o'X)   --> " ", bs, "circ (", axiom2texBC(X), ")".
axiom2texR2I('*1'X)  --> " ", bs, "star_1 (", axiom2texBC(X), ")".
axiom2texR2I('*2'X)  --> " ", bs, "star_2 (", axiom2texBC(X), ")".
axiom2texR2I(X & Y)  --> "(", axiom2texR2A(X), " ", bs, "land ", axiom2texP2A(Y), ")".
axiom2texR2I(X & Y)  --> "(", axiom2texP2A(X), " ", bs, "land ", axiom2texR2A(Y), ")".
axiom2texR2I(X v Y)  --> "(", axiom2texR2O(X), " ", bs, "lor ", axiom2texP2O(Y), ")".
axiom2texR2I(X v Y)  --> "(", axiom2texP2O(X), " ", bs, "lor ", axiom2texR2O(Y), ")".
axiom2texR2I(X -> Y) --> "(", axiom2texR2I(X), " ", bs, "ra ", axiom2texP2I(Y), ")".
axiom2texR2I(X -> Y) --> "(", axiom2texP2I(X), " ", bs, "ra ", axiom2texR2I(Y), ")".

%% P2 - divided into And, Or, Implication (for the bracketing)

axiom2texP2A(X)	     --> axiom2texC(X).
%axiom2texP2A(-X)     --> " ", bs, "lnot ", axiom2texC(X).
%axiom2texP2A('o'X)   --> " ", bs, "circ ", axiom2texC(X).
axiom2texP2A('*1'X)  --> " ", bs, "star_1 ", axiom2texC(X).
axiom2texP2A('*2'X)  --> " ", bs, "star_2 ", axiom2texC(X).
axiom2texP2A(X & Y)  --> axiom2texP2A(X), " ", bs, "land ", axiom2texP2A(Y).
axiom2texP2A(X & Y)  --> axiom2texP2A(X), " ", bs, "land ", axiom2texP2A(Y).
axiom2texP2A(X v Y)  --> "(", axiom2texP2O(X), " ", bs, "lor ", axiom2texP2O(Y), ")".
axiom2texP2A(X v Y)  --> "(", axiom2texP2O(X), " ", bs, "lor ", axiom2texP2O(Y), ")".
axiom2texP2A(X -> Y) --> "(", axiom2texP2I(X), " ", bs, "ra ", axiom2texP2I(Y), ")".
axiom2texP2A(X -> Y) --> "(", axiom2texP2I(X), " ", bs, "ra ", axiom2texP2I(Y), ")".

axiom2texP2O(X)	     --> axiom2texC(X).
%axiom2texP2O(-X)     --> " ", bs, "lnot ", axiom2texC(X).
%axiom2texP2O('o'X)   --> " ", bs, "circ ", axiom2texC(X).
axiom2texP2O('*1'X)  --> " ", bs, "star_1 ", axiom2texC(X).
axiom2texP2O('*2'X)  --> " ", bs, "star_2 ", axiom2texC(X).
axiom2texP2O(X & Y)  --> "(", axiom2texP2A(X), " ", bs, "land ", axiom2texP2A(Y), ")".
axiom2texP2O(X & Y)  --> "(", axiom2texP2A(X), " ", bs, "land ", axiom2texP2A(Y), ")".
axiom2texP2O(X v Y)  --> axiom2texP2O(X), " ", bs, "lor ", axiom2texP2O(Y).
axiom2texP2O(X v Y)  --> axiom2texP2O(X), " ", bs, "lor ", axiom2texP2O(Y).
axiom2texP2O(X -> Y) --> "(", axiom2texP2I(X), " ", bs, "ra ", axiom2texP2I(Y), ")".
axiom2texP2O(X -> Y) --> "(", axiom2texP2I(X), " ", bs, "ra ", axiom2texP2I(Y), ")".

axiom2texP2I(X)	     --> axiom2texC(X).
%axiom2texP2I(-X)     --> " ", bs, "lnot ", axiom2texC(X).
%axiom2texP2I('o'X)   --> " ", bs, "circ ", axiom2texC(X).
axiom2texP2I('*1'X)  --> " ", bs, "star_1 ", axiom2texC(X).
axiom2texP2I('*2'X)  --> " ", bs, "star_2 ", axiom2texC(X).
axiom2texP2I(X & Y)  --> "(", axiom2texP2A(X), " ", bs, "land ", axiom2texP2A(Y), ")".
axiom2texP2I(X & Y)  --> "(", axiom2texP2A(X), " ", bs, "land ", axiom2texP2A(Y), ")".
axiom2texP2I(X v Y)  --> "(", axiom2texP2O(X), " ", bs, "lor ", axiom2texP2O(Y), ")".
axiom2texP2I(X v Y)  --> "(", axiom2texP2O(X), " ", bs, "lor ", axiom2texP2O(Y), ")".
axiom2texP2I(X -> Y) --> "(", axiom2texP2I(X), " ", bs, "ra ", axiom2texP2I(Y), ")".
axiom2texP2I(X -> Y) --> "(", axiom2texP2I(X), " ", bs, "ra ", axiom2texP2I(Y), ")".

%% atoms / bound by 1 connective / 2 connectives
axiom2texC(a) 	    --> bs,"alpha".
axiom2texC(b) 	    --> bs,"beta".
axiom2texCa(a) 	    --> bs,"alpha".
axiom2texCb(b) 	    --> bs,"beta".
%axiom2texUC(-X)	    --> " ", bs,"lnot ", axiom2texC(X).
%axiom2texUC('o'X)   --> " ", bs, "circ ", axiom2texC(X).
axiom2texUC('*1'X)  --> " ", bs, "star_1 ", axiom2texC(X).
axiom2texUC('*2'X)  --> " ", bs, "star_2 ", axiom2texC(X).
%axiom2texUCa(-a)    --> " ", bs,"lnot ", bs, "alpha".
%axiom2texUCa('o'a)  --> " ", bs, "circ ", bs, "alpha".
axiom2texUCa('*1'a) --> " ", bs, "star_1 ", bs, "alpha".
axiom2texUCa('*2'a) --> " ", bs, "star_2 ", bs, "alpha".
%axiom2texUCb(-b)    --> " ", bs,"lnot ", bs, "beta".
%axiom2texUCb('o'b)  --> " ", bs, "circ ", bs, "beta".
axiom2texUCb('*1'b) --> " ", bs, "star_1 ", bs, "beta".
axiom2texUCb('*2'b) --> " ", bs, "star_2 ", bs, "beta".
axiom2texBC(X & Y)  --> axiom2texC(X), " ", bs, "land ", axiom2texC(Y).
axiom2texBC(X v Y)  --> axiom2texC(X), " ", bs, "lor ", axiom2texC(Y).
axiom2texBC(X -> Y) --> axiom2texC(X), " ", bs, "ra ", axiom2texC(Y).


%% test if a expression is the one that appears in the conclusion

%isLogical(-X) 	  --> isLogicalF(X).
%isLogical('o'X)	  --> isLogicalF(X).
isLogical('*1'X)  --> isLogicalF(X).
isLogical('*2'X)  --> isLogicalF(X).
%isLogicalF(-X) 	  --> axiom2texC(X).
%isLogicalF('o'X)  --> axiom2texC(X).
isLogicalF('*1'X) --> axiom2texC(X).
isLogicalF('*2'X) --> axiom2texC(X).
isLogicalF(X & Y) --> axiom2texC(X), axiom2texC(Y).
isLogicalF(X v Y) --> axiom2texC(X), axiom2texC(Y).
isLogicalF(X -> Y)--> axiom2texC(X), axiom2texC(Y).
isAtom(X)	--> axiom2texC(X).

print_phrase(V) :-
   phrase(V, S),
   name(A, S),
   write(A).

%%% Method:
%%% 	CheckCondition(+Formula, +Axiom, +Flag, -Condition)
%%% Functionality:
%%%	checks whether the formula \phi is in a 'good' position in the axiom, i.e.
%%%	\phi must not be contained in
%%%	(i) a positively occurring (sub)formula of the form psi1 & psi2, and
%%%	(ii) a negatively occurring (sub)formula of the form psi1 v psi2 
%%%	     or psi1 -> psi2.
%%% Parameters
%%% 	+Formula 	... formula \phi
%%% 	+Axiom		... the axiom/subformula containing \phi
%%%  	+Flag		... Even ( = 0), Odd ( = 1)
%%% 	-Condition	... 1 if \phi is positioned correctly, 0 otherwise
checkCondition(Formula, X & Y, Flag, Condition) :-
	CFlag = Flag mod 2,
	( CFlag > 0 ->
	  Condition = 0
	; isSubformulae(X, SubFormulaeX),
          ( member(Formula,SubFormulaeX) ->
	    checkCondition(Formula, X, Flag mod 2, Condition)
	  ; isSubformulae(Y, SubFormulaeY),
	    ( member(Formula, SubFormulaeY) ->
		checkCondition(Formula, Y, Flag mod 2, Condition)
	    )
	  )
	).
checkCondition(Formula, X v Y, Flag, Condition) :-
	CFlag = Flag mod 2,
	( CFlag < 1 ->
	  Condition = 0
	; isSubformulae(X, SubFormulaeX),
          ( member(Formula,SubFormulaeX) ->
	    checkCondition(Formula, X, (Flag) mod 2, Condition)
	  ; isSubformulae(Y, SubFormulaeY),
	    ( member(Formula, SubFormulaeY) ->
		checkCondition(Formula, Y, (Flag) mod 2, Condition)
	    )
	  )
	).
checkCondition(Formula, X -> Y, Flag, Condition) :-
	CFlag = Flag mod 2,
	( CFlag < 1 ->
	  Condition = 0
	; isSubformulae(X, SubFormulaeX),
          ( member(Formula,SubFormulaeX) ->
	    checkCondition(Formula, X, (Flag+1) mod 2, Condition)
	  ; isSubformulae(Y, SubFormulaeY),
	    ( member(Formula, SubFormulaeY) ->
	     	checkCondition(Formula, Y, Flag mod 2, Condition)
	    )
	  )
	).
checkCondition(_,Axiom,_,1) :-
	isUnaryAtomic(Axiom).
	
%%% Method:
%%%	isSubformulae(+Formula, -Subformulae)
%%% Functionality:
%%%	computes the subformulae of a given formula
%%% Parameters
%%%	+Formula	... formula whose subformulas will be computed
%%%	-Subformulae	... list of subformulae of the Formula
isSubformulae(X & Y, SF) :-
	isSubformulae(X, SF1),
	isSubformulae(Y, SF2),
	append(SF2, SF1, SF3),
	append([X & Y], SF3, SF).
isSubformulae(X v Y, SF) :-
	isSubformulae(X, SF1),
	isSubformulae(Y, SF2),
	append(SF2, SF1, SF3),
	append([X v Y], SF3, SF).
isSubformulae(X -> Y, SF) :-
	isSubformulae(X, SF1),
	isSubformulae(Y, SF2),
	append(SF2, SF1, SF3),
	append([X -> Y], SF3, SF).
isSubformulae('*1' X, SF) :-
	isSubformulae(X, SF1),
	append(['*1' X], SF1, SF).
isSubformulae('*2' X, SF) :-
	isSubformulae(X, SF1),
	append(['*2' X], SF1, SF).
isSubformulae(X, [X]).

isSubformulaeAxioms(Ax1 ';' Ax2, Axioms, AxiomSF) :-
	isSubformulaeAxioms(Ax2, Axioms2, AxiomSF2),
	isSubformulae(Ax1, AxiomSF1),
	append([Ax1], Axioms2, Axioms),
	append([AxiomSF1], AxiomSF2, AxiomSF).
isSubformulaeAxioms(Axiom, [Axiom], [AxiomSF]) :-
	isSubformulae(Axiom, AxiomSF).

%%% Method:
%%%	isUnaryAtomic(+Formula)
%%% Functionality:
%%%	checks whether the formula is a propositional variable (i.e. atomic)
%%%	or if it is a propositional variable / subformula preceded by a 
%%%	unary connective
%%% Parameters
%%%	+Formula	... formula that will be checked
isUnaryAtomic(Formula) :-
	atomic(Formula).
isUnaryAtomic('*1' _).
isUnaryAtomic('*2' _).
	
%%% Method:
%%%	axiomCheckCondition(+Axiom, +AxiomSF, -Condition)
%%% Functionality
%%%	checks for the variable (\phi) that will be in the conclusion of the 
%%%	rule, whether it is positioned in correctly
%%% Parameters
%%%	+Axiom 		... axiom that will be computed
%%%	+AxiomSF 	... subformulas of the axiom
%%%	-Condition 	... 1 if variable is positioned correctly, 0 otherwise
axiomCheckCondition(Axiom, AxiomSF, Condition) :-
	X = '*1'('*1' _),
	member(X, AxiomSF),
	checkCondition(X, Axiom, 1, Condition).
axiomCheckCondition(Axiom, AxiomSF, Condition) :-
	X = '*1'('*2' _),
	member(X, AxiomSF),
	checkCondition(X, Axiom, 1, Condition).
axiomCheckCondition(Axiom, AxiomSF, Condition) :-
	X = '*2'('*1' _),
	member(X, AxiomSF),
	checkCondition(X, Axiom, 1, Condition).
axiomCheckCondition(Axiom, AxiomSF, Condition) :-
	X = '*2'('*2' _),
	member(X, AxiomSF),
	checkCondition(X, Axiom, 1, Condition).
axiomCheckCondition(Axiom, AxiomSF, Condition) :-
        X = '*1'(_ & _),
        member(X, AxiomSF),
        checkCondition(X, Axiom, 1, Condition).
axiomCheckCondition(Axiom, AxiomSF, Condition) :-
        X = '*2'(_ & _),
        member(X, AxiomSF),
        checkCondition(X, Axiom, 1, Condition).
axiomCheckCondition(Axiom, AxiomSF, Condition) :-
        X = '*1'(_ v _),
        member(X, AxiomSF),
        checkCondition(X, Axiom, 1, Condition).
axiomCheckCondition(Axiom, AxiomSF, Condition) :-
        X = '*2'(_ v _),
        member(X, AxiomSF),
        checkCondition(X, Axiom, 1, Condition).
axiomCheckCondition(Axiom, AxiomSF, Condition) :-
        X = '*1'(_ -> _),
        member(X, AxiomSF),
        checkCondition(X, Axiom, 1, Condition).
axiomCheckCondition(Axiom, AxiomSF, Condition) :-
        X = '*2'(_ -> _),
        member(X, AxiomSF),
        checkCondition(X, Axiom, 1, Condition).
axiomCheckCondition(Axiom, AxiomSF, Condition) :-
        X = '*1'_,
        member(X, AxiomSF),
        checkCondition(X, Axiom, 1, Condition).
axiomCheckCondition(Axiom, AxiomSF, Condition) :-
        X = '*2'_,
        member(X, AxiomSF),
        checkCondition(X, Axiom, 1, Condition).

axiomCheckConditions([], [], 1).
axiomCheckConditions([H|T], [HSF|TSF], Condition) :-
        axiomCheckCondition(H, HSF, Condition1),
        ( Condition1 = 0 ->
          Condition = 0
        ; axiomCheckConditions(T, TSF, Condition)
        ).

%%% Unit-Tests checkCondition
?- \+ checkCondition('*1'('*1' a), (('*1'('*1' a) -> a) -> '*1' a), 1, 1).
?- checkCondition('*1'('*1' a), a -> ('*1' a -> '*1'('*1' a)), 1, 1).
?- checkCondition('*1'('*1' a), a -> ('*1'('*1' a) -> a), 1, 1).
?- \+ checkCondition('*1'('*1' a), (a -> '*1'('*1' a)) -> '*1' a, 1, 1).
?- checkCondition('*1'('*1' a), ('*1'('*1' a) & a) -> '*1' a, 1, 1).
?- \+ checkCondition('*1'('*1' a), (('*1'('*1' a) & a) -> '*1' a) -> a, 1, 1).
?- checkCondition('*1'('*1' a), ('*1'('*1' a) & a) -> ('*1' a -> a), 1, 1).
?- checkCondition('*1'('*1' a), '*1' a -> (('*1'('*1' a) & a) -> a), 1, 1).
?- \+ checkCondition('*1'('*1' a), (('*1'('*1' a) & a) v '*1' a) -> a, 1, 1).
?- \+ checkCondition('*1'('*1' a), '*1'('*1' a) & a, 1, 1).
?- checkCondition('*1'('*1' a), '*1'('*1' a) v a, 1, 1).
?- checkCondition('*1' a, a v '*1' a, 1, 1).
?- checkCondition('*1'('*1' a), '*1'('*1' a) -> a, 1, 1).
?- checkCondition('*1'(a & b), '*1'(a & b) -> ('*1' a v '*1' b), 1, 1).
?- checkCondition('*1'(a v b), '*1'(a v b) -> ('*1' a & '*1' b) ,1, 1).
?- checkCondition('*1'(a -> b), '*1'(a -> b) -> (a & '*1' b) ,1, 1).
?- checkCondition('*1' a, a -> ('*1' a -> b) ,1, 1).
?- checkCondition('*1'('*1' a), a -> '*1'('*1' a) ,1, 1).
?- checkCondition('*1'(a & b), ('*1' a v '*1' b) -> '*1'(a & b), 1, 1).
?- checkCondition('*1'(a v b), ('*1' a & '*1' b) -> '*1'(a v b), 1, 1).
?- checkCondition('*1'(a -> b), (a & '*1' b) -> '*1'(a -> b), 1, 1).
?- checkCondition('*2' a, a -> ('*1' a -> ('*2'a -> b)), 1, 1).
?- checkCondition('*1' a, a -> ('*1' a -> ('*2'a -> b)), 1, 1).
?- checkCondition('*2'(a v b), '*2' (a v b) -> ('*2'a v '*2'b) ,1, 1).
?- checkCondition('*2' a, '*2' a v a ,1, 1).
?- checkCondition('*2'a, '*2' a v '*1' a ,1, 1).
?- checkCondition('*1'a, '*2' a v '*1' a ,1, 1).
?- checkCondition('*1'('*2' a), '*1'('*2' a) -> a ,1, 1).
?- checkCondition('*1'('*2' a), '*1'('*2' a) -> '*1' a ,1, 1).
?- checkCondition('*2'(a & b), '*2' a -> '*2'(a & b) ,1, 1).
?- checkCondition('*2'(a & b), '*2' b -> '*2'(a & b) ,1, 1).
?- checkCondition('*2'(a -> b), ('*2'a & '*2'b) -> '*2'(a -> b) ,1, 1).
?- checkCondition('*2'('*1' a), '*2' a -> '*2'('*1' a) ,1, 1).
?- \+ checkCondition('*2'(a & b), (('*2' a & '*2' b) -> ('*2'(a & b) & (a v b))), 1,1).
?- \+ checkCondition('*2'(a & b), (('*2' a & '*2' b) -> ('*2'(a & b) & (a v b))), 1, 1).
?- checkCondition('*2' a, '*2' a & '*2' b -> (a v b), 1, 1).
?- checkCondition('*1'('*1' a), '*1'('*1' a) v a, 1, 1).
?- \+ checkCondition('*1'('*1' a), '*1'('*1' a) & a, 1, 1).
?- \+ checkCondition('*1'('*1' a), ('*1'('*1' a) -> a) & a, 1, 1).
?- checkCondition('*1'('*1' a), ('*1'('*1' a) -> '*1' a) v a, 1, 1).
?- checkCondition('*1'('*1' a), a -> (('*1'('*1' a) -> '*1' a) v a), 1, 1).
?- \+ checkCondition('*1'('*1' a), a & ('*1' a & '*1'('*1' a)), 1, 1).
?- \+ checkCondition('*1'('*1' a), a & ('*1' a v '*1'('*1' a)), 1, 1).
?- \+ checkCondition('*1' a, a v ('*1' a & a), 1, 1).
?- checkCondition('*1' a, a v ('*1' a -> '*1' a), 1, 1).
?- checkCondition('*1' a, '*1' a v ('*1' a -> '*1' a), 1, 1).
?- checkCondition('*1' a, '*1' a -> ('*1' a -> '*1' a), 1, 1).
?- \+ checkCondition('*1' a, (a -> '*1' a) -> '*1' b, 1, 1).
?- checkCondition('*1' a, (a v '*1' a), 1, 1).
?- checkCondition('*1' a, ('*1' b -> ('*1' a -> a)) v ('*2' a -> '*1' a), 1, 1).