%%% Filename : p_kernl.pl
%%% Date     : 28.03.2012
%%% Contents : main method to start the program
%%% Author   : Lara Katharina Spendier <lara@logic.at>

:- use_module(library(time)).
:- ['p_operators.pl'].
:- ['p_listfunctions.pl'].
:- ['p_invertible.pl'].
:- ['p_completion.pl'].
:- ['p_printcmd.pl'].
:- ['p_grammar.pl'].
:- ['p_tex.pl'].
:- ['p_axiom2rule.pl'].
:- ['p_initialTT.pl'].
:- ['p_matrices.pl'].
:- ['p_unaryrules.pl'].
:- ['p_encoding.pl'].

compute :-
		read(Axiom),
		read(Connective),
		(\+ axioms2tex(Axiom,_,[]) ->
		handle("1")
		; 
		isSubformulaeAxioms(Axiom, Axioms, AxiomSF),
		axiomCheckConditions(Axioms, AxiomSF, Condition),
	 	( Condition = 0 ->
		  handle("1")
		; writef('\nComputing axiom...\n'),
		  (Connective = 0 ->
		      catch(			
		      call_with_time_limit(1, startComputation(Axiom, 0, 3, _, 0)),  % 0 predefined unary connective (LK+)
           	      Exception,                     
            	      handle(Exception))
		    ; (Connective = 1 ->
		        catch(
            	        call_with_time_limit(1, startComputation(Axiom, 1, 2, _, 0)), % 1 predefined unary connective (testing)
            	        Exception,                     
            	        handle(Exception))
		      ; catch(
			call_with_time_limit(1, startComputation(Axiom, 2, 1, 10, 1)), % 2 predefined unary connectives (BK)
            	        Exception,                     
            	        handle(Exception))
		)))).

%% computeweb:	+Axiom: axiom given by the user; 0 if no axiom or incorrect string
%%		+Connective: basic calculus chosen by the user; 0 = LK+, 1 = BK
%%		+N: number of unary connectives occurring in the axiom
computeweb(0, Connective, _) :-
	catch(
	call_with_time_limit(1, startComputation(0, Connective)),  % 0 predefined unary connective (LK+)
        Exception,                     
        handle(Exception)).

computeweb(Axiom, Connective, N) :-
		(\+ axioms2tex(Axiom,_,[]) ->
		  handle("1")
		; isSubformulaeAxioms(Axiom, Axioms, AxiomSF),
		  axiomCheckConditions(Axioms, AxiomSF, Condition),
	 	  ( Condition = 0 ->
		    handle("1")
		  ;  writef('\nComputing axiom...\n'),
		    (Connective = 0 ->
		      catch(
		      call_with_time_limit(1, startComputation(Axiom, 0, N, _, 0)),  % 0 predefined unary connective (LK+)
           	      Exception,                     
            	      handle(Exception))
		    ; catch(
            	      call_with_time_limit(1, startComputation(Axiom, 2, 1, 10, 1)), % 2 predefined unary connectives (BK)
            	      Exception,                     
            	      handle(Exception))
		))).

handle(_) :-
		writef('\n\nThe axiom you provided was not computable.'),
		texOut(100,_, _).
		


%% Axiom-Tests of the grammar
%% Axiom Tests Figure 1
?- axiom2tex(a v '*1' a ,_,[]). % (n1)
?- axiom2tex('*1'('*1' a) -> a ,_,[]). % (c)
?- axiom2tex('*1'(a & b) -> ('*1' a v '*1' b) ,_,[]).
?- axiom2tex('*1'(a v b) -> ('*1' a & '*1' b) ,_,[]).
?- axiom2tex('*1'(a -> b) -> (a & '*1' b) ,_,[]).
?- axiom2tex(a -> ('*1' a -> b) ,_,[]).
?- axiom2tex(a -> '*1'('*1' a) ,_,[]).
?- axiom2tex(('*1' a v '*1' b) -> '*1'(a & b) ,_,[]).
?- axiom2tex(('*1' a & '*1' b) -> '*1'(a v b) ,_,[]).
?- axiom2tex((a & '*1' b) -> '*1'(a -> b) ,_,[]).
?- axiom2tex(a -> ('*1' a -> ('*2'a -> b)) ,_,[]).
?- axiom2tex('*2' (a v b) -> ('*2'a v '*2'b) ,_,[]).
?- axiom2tex('*2' a v a ,_,[]). 
?- axiom2tex('*2' a v '*1' a ,_,[]).
?- axiom2tex('*1'('*2' a) -> a ,_,[]).
?- axiom2tex('*1'('*2' a) -> '*1' a ,_,[]).  
?- axiom2tex('*2' a -> '*2'(a & b) ,_,[]).
?- axiom2tex('*2' b -> '*2'(a & b) ,_,[]).
?- axiom2tex(('*2'a & '*2'b) -> '*2'(a -> b) ,_,[]).
?- axiom2tex('*2' a -> '*2'('*1' a) ,_,[]).  

%% Artificial Tests
?- \+ axiom2tex(a,_,[]).
?- \+ axiom2tex(a v b, _, []).
?- axiom2tex('*2'a, _, []).
?- axiom2tex('*2'(a v b), _, []).
?- axiom2tex(a -> '*1' b, _, []).
?- axiom2tex('*1'('*1' a) -> a, _, []).
?- axiom2tex(a -> '*1'('*1' a), _, []).
?- axiom2tex(('*1' a v '*1' b) -> '*1'(a & b), _, []).
?- axiom2tex(('*1' a & '*1' b) -> '*1'(a v b), _, []).
?- axiom2tex('*1'(a v b) -> ('*1' a & '*1' b), _, []).
?- axiom2tex('*1'(a & b) -> ('*1' a v '*1' b), _, []).
?- axiom2tex((a & '*1' b) -> '*1'(a -> b), _, []).
?- \+ axiom2tex('*1'(a & b) -> '*1'(a & '*1' b), _, []).
?- axiom2tex(('*1' a v '*1'(a&b)) -> (a & b), _, []).
?- axiom2tex(('*1' a v '*1' b) -> ('*1' a v '*1'(a&b)), _, []).
?- \+ axiom2tex(('*1' a v '*1'(a&b)) -> ('*1' a v '*1'(a&b)), _, []).
?- \+ axiom2tex(('*1'(a&b) v '*1'(a&b)) -> ('*1' a v '*1'(a&b)), _, []).
?- \+ axiom2tex(('*1'(a&b) v '*1'(a&b)) -> (a v b), _, []).
?- axiom2tex(('*1'(a&b) v '*1' b) -> (a v b), _, []).
?- axiom2tex((a v '*1'(a&b)) -> (a v b), _, []).
?- \+ axiom2tex('*1'('*2'a) -> (a & '*1' b), _, []).
?- \+ axiom2tex('*2'('*2'a) -> (a & '*1' b), _, []).
?- \+ axiom2tex('*2'('*1' a) -> (a & '*1' b), _, []).
?- axiom2tex('*1'('*2'a) -> (a & '*1' a), _, []).
?- axiom2tex('*2'('*2'a) -> (a & '*1' a), _, []).
?- axiom2tex('*2'('*1' a) -> (a & '*1' a), _, []).
?- axiom2tex(('*2'a & '*2'b) -> '*2'(a v b), _, []).
?- axiom2tex(('*2'a & '*2'b) -> '*2'(a -> b), _, []).
?- axiom2tex('*2'a -> '*2'(a & b), _, []).
?- \+ axiom2tex(('*2'a v '*2'(a&b)) -> ('*2'a v '*2'(a&b)), _, []).
?- \+ axiom2tex(('*2'(a&b) v '*2'(a&b)) -> ('*2'a v '*2'(a&b)), _, []).
?- \+ axiom2tex(('*2'(a&b) v '*2'(a&b)) -> (a v b), _, []).
?- axiom2tex('*2' a -> ('*2' a v '*2'(a&b)), _, []).
?- \+ axiom2tex('*2'a -> ('*2'(a&b) v '*2'(a&b)), _, []).
?- axiom2tex('*2'a -> ('*2'a v (b & '*2'(a&b))), _, []).
?- axiom2tex('*2'a -> ('*2'a v ('*2'(a&b) &  b)), _, []).
?- \+ axiom2tex('*1'('*1'('*2'a)) -> ('*1'a), _, []).
?- \+ axiom2tex('*1'('*1'('*2'a)) -> ('*1'a & '*1' b), _, []).
?- \+ axiom2tex('*2'('*2'a) -> ('*2'a & '*1'('*1'a)), _, []).
?- \+ axiom2tex('*2'('*2'('*1' a)) -> ('*1'a & '*1' b), _, []).
?- \+ axiom2tex('*2'(a & b) -> '*2'(a v b), _, []).
?- axiom2tex(('*2'a & b) -> '*1'(a v b), _, []).
?- \+ axiom2tex('*1'('*2'('*1'a) & '*2'b) -> '*2'(a -> b), _, []).
?- \+ axiom2tex(('*2'a v '*2'(a&b)) -> ('*2'('*1'a) v '*2'b), _, []).
?- \+ axiom2tex('*2'('*2'a) -> ('*2' a v '*2'(a&b)), _, []).
?- axiom2tex('*2'a -> ('*2'a v ('*2'b & '*2'(a&b))), _, []).
?- \+ axiom2tex('*2'a -> ('*2'a v ('*2'('*1'b) & b)), _, []).
?- \+ axiom2tex('*2'b -> ('*2'b v ('*2'('*1'a) & a)), _, []).
?- axiom2tex('*2'b -> ('*2'b v ('*2'('*1'b) &  b)), _, []).
?- axiom2tex('*2'a -> ('*2'a v ('*2'('*1'a) &  a)), _, []).
?- axiom2tex(('*1' a & '*2' b) -> '*1'(a v b), _, []).
?- axiom2tex('*1' a -> '*1'(a & b), _, []).
?- \+ axiom2tex('*1'a -> ('*1'(a&b) v '*1'(a&b)), _, []).
?- \+ axiom2tex(('*1' a & '*1' b) -> ('*1' b & '*1' a), _, []).

%% Axiom-Tests of logical rules
?- isLogical('*1'('*1' a), _, []).
?- isLogical('*2'('*1' a), _, []).
?- isLogical('*2'(a v b), _, []).
?- isLogical('*1'(a -> b), _, []).
?- isLogical('*1'('*2'b), _, []).
?- \+ isLogical('*1' a, _, []).
?- \+ isLogical(a v b, _, []).
?- \+ isLogical('*1'('*1' a -> b), _, []).
?- \+ isLogical('*1' b, _, []).
?- isLogicalF('*1' a, _, []).
?- isLogicalF('*2'a, _, []).
?- \+ isLogicalF('*1'('*1' a), _, []).
