%%% Filename : p_listfunctions.pl
%%% Date     : 23.02.2011
%%% Contents : helper methods for list manipulation
%%% Author   : Lara Katharina Spendier <lara@logic.at>

%% append(+Element, +List, -List)
%%		appends an element to a list
append([],L,L).
append([H|T],L2,[H|L3]) :- append(T,L2,L3).

%% remove(+Element, +List, -List)
%%		removes an element from a list
remove(X, L2, L3) :-
	\+ member(X, L2),
	append([], L2, L3).

remove(H, [H|L2], L3) :-
	append(L2, [], L3).

remove(X, [H|L2], L3) :-
	X \= H,
	member(X, L2),
	remove(X, L2, L4),
	append([H], L4, L3).

%% removeList(+ListOfElements, +List, -List)
%%		removes a list of elements from a list
removeList([], L, L).
removeList([H|T], L, LR) :-
	remove(H, L, L1),
	removeList(T, L1, LR).
	
%% remove_double(+List, -List)
%%		removes elements that occur twice from a list
remove_double([], L, L).
remove_double([H|T], L2, Result) :-
	\+ member(H, L2),
	append([H], L2, L3),
	remove_double(T, L3, Result).
remove_double([H|T], L2, Result) :-
	member(H, L2),
	remove_double(T, L2, Result).

remove_once([], Remainder, Remainder).
remove_once([Current|Delete], List, X) :-
    select(Current, List, Rest),
    remove_once(Delete, Rest, X).

%%  reverseList
%%		reverses the list
reverseList([],A,A).
reverseList([H|T],A,R) :- reverseList(T,[H|A],R).

%% replacElements(+List, +ReplaceElement, +NewElement, -List)
%%		replaces an element with a new element (which is a list)
replaceElements([], _, _, []).

replaceElements([R|T], R, E, NL1) :-
	replaceElements(T, R, E, NL0),
	append(E, NL0, NL1).
	
replaceElements([H|T], R, E, NL1) :-
	H \= R,
	replaceElements(T, R, E, NL0),
	append([H], NL0, NL1).

%% elementAt (+Index, +List, -Element)
%%		gets the element at position Index of the List

elementAt(0,[X|_],X).
elementAt(K,[_|L],X) :- K > 0, K1 is K - 1, elementAt(K1,L,X).
