%%% Filename : p_matrices.pl
%%% Date     : 13.04.2012
%%% Contents : computes the matrices out of the axioms
%%% Author   : Lara Katharina Spendier <lara@logic.at>

%% is_matrices_base(+N, +Star1, +Star2)
%%	computes the truth values for the base calculus
%%	Number 		... Number of new unary connectives 
%%			    1 ... 1 new unary connective, baseCalculus = 4 TV
%%			    2 ... 2 new unary connectives	
%%	[Star1,Star2] 	... ID for the new unary connectives; 
%%				1 ... negation, 10 ... circleis_matrices_base(1, S1, _).
is_matrices_base(0, 1, _, Logical, MTV, RuleR) :-
	isTruthTable1(Val, TVal, And, Or, Implies, Star1, []),
	MatrixTV = [Val, TVal, And,Or,Implies,Star1,[]], 
	computeVM(MatrixTV, Logical, VM),
	MatrixTV1 = [Val, VM, And, Or, Implies, Star1, []],
	MTV = MatrixTV1,
	RuleR = [].

is_matrices_base(0, _, _, Logical, MTV, RuleR) :-
	isTruthTable2(Val, TVal, And, Or, Implies, Star1, Star2),
	MatrixTV = [Val, TVal, And,Or,Implies,Star1,Star2], 
	computeVM(MatrixTV, Logical, VM),
	MatrixTV1 = [Val, VM, And, Or, Implies, Star1, Star2],
	MTV = MatrixTV1,
	RuleR = [].

is_matrices_base(1, S1, _, Logical, MTV, RuleR) :-
	isTruthTable1(Val, TVal, And, Or, Implies, Star1, []),
	MatrixTV = [Val, TVal, And,Or,Implies,Star1,[]],
	isUnaryConnective(S1, '*1', Rule),
	computeVM(MatrixTV, Rule, VM),
	computeVM(MatrixTV, Logical, VM1),
	getSameVals(VM, VM1, VM2),
	MatrixTV1 = [Val, VM2, And, Or, Implies, Star1, []],
	MTV = MatrixTV1,
	RuleR = [Rule].

is_matrices_base(2, S1, S2, Logical, MTV, RuleR) :-
	isTruthTable2(Val, TVal, And, Or, Implies, Star1, Star2),
	MatrixTV = [Val, TVal, And,Or,Implies,Star1,Star2], 
	isUnaryConnective(S1, '*1', Rule1),
	isUnaryConnective(S2, '*2', Rule2),
	computeVM(MatrixTV, Rule1, VM),
	MatrixTV1 = [Val, VM, And, Or, Implies, Star1, Star2],
	computeVM(MatrixTV1, Rule2, VM1),
	computeVM(MatrixTV1, Logical, VM2),
	getSameVals(VM1, VM2, VM3),
	MatrixTV2 = [Val, VM3, And, Or, Implies, Star1, Star2],
	MTV = MatrixTV2,
	RuleR = [Rule1, Rule2].

%% isUnaryConnective(+Nr, +C, -Rule)
%%	gets the truth table for the connective C with the Id Nr
isUnaryConnective(ConnectiveNr, Connective, Rule) :-
	isUnaryRule(ConnectiveNr, Connective, Rule).
%	nl, nl, write('New Unary Connective Rule:'),
%	printrule(Rule).

%% is_matrices_unary
is_matrices_unary(MatrixTV, LogicalRules, MTV) :-
	MatrixTV = [Val, TVal, And, Or, Implies, _, []], 
	isUnaryTruthTable(TVal, '*1', Star11),
	computeLVM(Star11, LogicalRules, '*1', Star12),
	computeBLVM(And, TVal, LogicalRules, '&', And1),
	computeBLVM(Or, TVal, LogicalRules, 'v', Or1),
	computeBLVM(Implies, TVal, LogicalRules, '->', Implies1),
	MTV = [Val, TVal, And1, Or1, Implies1, Star12, []].

is_matrices_unary(MatrixTV, LogicalRules, MTV) :-
	MatrixTV = [Val, TVal, And, Or, Implies, _, _], 
	isUnaryTruthTable(TVal, '*1', Star11),
	computeLVM(Star11, LogicalRules, '*1', Star12),
	isUnaryTruthTable(TVal, '*2', Star21),
	computeLVM(Star21, LogicalRules, '*2', Star22),
	computeBLVM(And, TVal, LogicalRules, '&', And1),
	computeBLVM(Or, TVal, LogicalRules, 'v', Or1),
	computeBLVM(Implies, TVal, LogicalRules, '->', Implies1),
	MTV = [Val, TVal, And1, Or1, Implies1, Star12, Star22].

%% isUnaryTruthTable
isUnaryTruthTable(TVal, Connective, NewTT) :-
	isUnaryTruthTableBase(TVal, TVal, Connective, NewTT1),
	NewTT = NewTT1.

isUnaryTruthTableBase([], _, _, []).
isUnaryTruthTableBase([H|T], TVal, Connective, NewTT) :-
	isUnaryTruthTableBase(T, TVal, Connective, NewTT1),
	isUVM(H, TVal, Connective, NewTT2),
	append([[H]], [NewTT2], NewTT3),
	append([NewTT3], NewTT1, NewTT).
	
isUVM(_, [], _, []).
isUVM(U1, [H|T], Connective, NewTT) :-
	isUVM(U1, T, Connective, NewTT2),
	( Connective = '*1' ->
	  Index = 1
	; Index = 2
	),
	elementAt(Index, U1, V1),
	elementAt(0, H, V2),
	( V1 = V2 ->
	  append([H], NewTT2, NewTT)
	; NewTT = NewTT2
	).


%% computeLVM(+TruthTable, +Rules, -TruthTable)
%%	computes the new truth tables according to the logical rules
%%	TruthTable	... Truth Table of the form [[[ U1, U2], [Values]], ...]
%%	Rules		... Logical Rules 
computeLVM([], _, _, []).
computeLVM([[U1, UCheck]|T], LRules, Connective, NewTT) :-
%	nl, writef('U1: %q -- UCheck: %q -- Connective %q', [U1, UCheck, Connective]),
	isLogicalVM(U1, UCheck, Connective, LRules, NewVM),
	computeLVM(T, LRules, Connective, NewTT2),
	append([[U1, NewVM]], NewTT2, NewTT).

computeBLVM([], _, _, _, []).
computeBLVM([[U, UCheck]|T], TVal, LRules, Connective, NewTT) :-
	U = [U1, U2],
%	nl, writef('BVM U: %q, U1: %q, U2: %q -- UCheck: %q -- Connective %q', [U, U1, U2, UCheck, Connective]),
	( (member(U1, TVal), member(U2, TVal)) ->
%	  nl, writef('U1 + U2 are members'),
	  isLogicalBVM(U1, U2, UCheck, TVal, Connective, LRules, NewVM),
	  Append = 1
	; Append = 0
	),
	computeBLVM(T, TVal, LRules, Connective, NewTT2),
	( Append = 1 ->
	  append([[[U1, U2], NewVM]], NewTT2, NewTT)
	  ; NewTT = NewTT2
	).


isLogicalVM(_, A, _, [], A).
isLogicalVM(U1, UCheck, Connective, [[P, [A,S]]|T], NewVM) :-
	isMainUConnective([A,S], MainUCon, MainUVar, B),
%	nl, writef('Rule Conclusion A: %q S %q MainCon %q MainVar %q', [A, S, MainUCon, MainUVar]),
	( B = 2 ->
	  UCheck1 = UCheck
	  ;
	  ( MainUCon = Connective ->
	    ( Connective = '*1' ->
	      Index = 1
	    ; Index = 2
	    ), 
	    % check <u1> |= Q or <u1,u2> |= Q
   	    %nl, writef('isRV: U1 %q -- UCheck %q -- B %q', [U1, UCheck, B]),
	    isRemoveVals(U1, UCheck, B, MainUVar, [P, [A,S]], Index, RVal),
	    removeList(RVal, UCheck, UCheck1)
	  ; UCheck1 = UCheck
	  )
	),
	isLogicalVM(U1, UCheck1, Connective, T, NewVM).

isLogicalBVM(_, _, A, _, _, [], A).
isLogicalBVM(U1, U2, UCheck, TVal, Connective, [[P, [A,S]]|T], NewVM) :-
%	nl, writef('BVM Rule Conclusion A: %q S %q // Premises %q', [A, S, P]),
	isMainBConnective([A,S], MainBCon, MainUCon, MainBVar1, MainBVar2, B),
%	nl, writef('BVM is member: BCon %q -- UCon %q -- B %q - Var1 %q - Var2 %q', [MainBCon, MainUCon, B, MainBVar1, MainBVar2]),
	( B = 2 ->
	  getSameVals(UCheck, TVal, UCheck1)
%	  UCheck1 = UCheck
%	  nl, writef('UCheck: %q -- TVal: %q -- UCheck1 %q', [UCheck, TVal, UCheck1])
	  ;
	  ( MainBCon = Connective ->
	    ( MainUCon = '*1' ->
	      Index = 1
	    ; Index = 2
	    ), 
%	    nl, writef('is Remove Vals!'),
	    % check <u1,u2> |= Q	  
	    isRemoveValsBinary(U1, U2, UCheck, B, MainBVar1, MainBVar2, [P, [A,S]], Index, RVal1),
%	    isRemoveVals([U1], UCheck, B, MainBVar1, [P, [A,S]], Index, RVal2),
%	    isRemoveVals([U2], UCheck, B, MainBVar2, [P, [A,S]], Index, RVal2),
%	    getSameVals(RVal1, RVal2, RVal),
%	    nl, writef('Rval1: %q  === RVAL: %q', [RVal1, RVal2, RVal]),
%	    removeList(RVal, UCheck, UCheck2), orig
%	    getSameVals(UCheck2, TVal, UCheck1) orig
	    removeList(RVal1, UCheck, UCheck2),
	    getSameVals(UCheck2, TVal, UCheck1)
%	    nl, writef('New UCheck1: %q', [UCheck1])
	  ; getSameVals(UCheck, TVal, UCheck1)
	  )
	),
	isLogicalBVM(U1, U2, UCheck1, TVal, Connective, T, NewVM).

getSameVals([],_,[]).
getSameVals([H|T], List, RVal) :-
	getSameVals(T, List, RVal1),
	( member(H, List) ->
	  append([H], RVal1, RVal)
	; RVal = RVal1
	).

%% computeVM(+MatrixTV, +Rule, -VM)
%%	computes the possible truth values for the base calculus plus the new connectives
%%	MatrixTV	... The list with all truth values for all connectives
%%	Rule 		... The rule(s) for the new unary connective
%%	VM		... the possible truth values for the logic
computeVM(_, [], []).

computeVM(MatrixTV, Rule, VM) :-
	Rule = [[P|[C]]|T],
	MatrixTV = [_, TVal, _, _, _, _, []],
	computeVM(MatrixTV, T, VM1),
	( VM1 = [] ->
	  Val1 = TVal
	; Val1 = VM1
	), 
%	nl, writef('H Star1: %q -- Val: %q', [C, Val1]),
	isMainConnective(C, '*1', Var, B),
%	determineB(C, B),
%	nl, writef('B is: %q', [B]),
	( B \= 2 ->
	  isVM(Val1, B, Var, [P|C], 1, VM)
	; VM = Val1
	).

computeVM(MatrixTV, Rule, VM) :-
	Rule = [[P|[C]]|T],
	MatrixTV = [_, TVal, _, _, _, _, _],
	computeVM(MatrixTV, T, VM1),
	( VM1 = [] ->
	  Val1 = TVal
	; Val1 = VM1
	), 
%	nl, writef('H Star2: %q -- Val: %q', [C, Val1]),
	%determineB(C, B),
	%isConnectives(C, B, Connective),
	isMainConnective(C, Connective, Var, B),
%	nl, writef('B is: %q -- connective %q -- Var %q', [B, Connective, Var]),
	( B \= 2 ->
	  ( Connective = '*1' -> %Connective = ['*1'] ->
	      isVM(Val1, B, Var, [P|C], 1, VM)
	    ; isVM(Val1, B, Var, [P|C], 2, VM)
	  )
	  ; VM = Val1
	).

%% determineB(+Ruleconclusion, -B)
%%	determines b (= 0,f or = 1,t) of the logical connective
%%	MatrixTV	... The list with all truth values for all connectives
%%	Rule 		... The rule(s) for the new unary connective
%%	VM		... the possible truth values for the logic
determineB([_, []] , 0).
determineB([[], _], 1).


%% isVM(+Val, +B, +Rule, +Index, -Val)
%%	determines the new possible truth values
isVM(Val, B, Var, [P|C], Index, Val1) :-
%	nl, writef('B is: %q -- Rule is %q', [B, [P|C]]),
	isRemoveVals(Val, B, Var, [P|C], Index, RVal),
	removeList(RVal, Val, Val1).

%% isRemoveVals(+Val, +B, +Conclusion, +Index, -RemoveValue)
%% isRemoveVals(+PremiseVal, +StarVal, +B, +Conclusion, +Index, -RemoveValue)
%%	determines which truth values have to be removed
%%	-> all values where u* != b (u \in V) and <u> |= Q (Q / {b:*p})
%%	PremiseVal ... <u> |= Q
%%	StarVal	   ... u* != b
isRemoveVals([], _, _, _, _, []).
isRemoveVals([H|T], B, Var, [P|C], Index, RVal) :-
	elementAt(Index, H, V),
	%nl,writef('Element at %q in %q is %q -- B %q', [Index, H, V, B]),
	( B \= V ->
%%% empty set of premises: remove!
	  %nl,writef('Element at %q in %q is %q -- B %q -- P %q -- Var %q', [Index, H, V, B, P, Var]),
	  removeNoSubFormula(Var, P, P1),
%	  nl, writef('P: %q --- P1: %q', [P, P1]),
	  ( (P = []; P1 = []) ->
	    Remove = 1
	  ; isRemoveValsPremises(H, P, Var, Remove)
	  ), 
	  ( Remove = 1 ->
	    RV = [H]
	  ; RV = []
	  )
	; RV = []
	),
	isRemoveVals(T, B, Var, [P|C], Index, RVal1),
	append(RV, RVal1, RVal).

isRemoveValsBinary(_, _, [], _, _, _, _, _, []).
isRemoveValsBinary(U1, U2, SVal, B, VarU1, VarU2, [P|C], Index, RVal) :-
	SVal = [H|T],
	elementAt(Index, H, V),
%	nl,writef('    Element at %q in %q is %q', [Index, H, V]),
	( B \= V ->
%	  write('    not same: RVP'),
%	  nl,writef('Element at %q in %q is %q -- B %q -- P %q', [Index, H, V, B, P]),
	  ( P = [] ->
	    Remove = 1
	  ; isRemoveValsPremisesBinary(U1, U2, P, VarU1, VarU2, Remove)
	  ),
	  ( Remove = 1 ->
	    RV = [H]
	  ; RV = []
	  )
	; RV = []
	),
	isRemoveValsBinary(U1, U2, T, B, VarU1, VarU2, [P|C], Index, RVal1),
	append(RV, RVal1, RVal).


isRemoveVals(_, [], _, _, _, _, []).
isRemoveVals(PVal, SVal, B, Var, [P|C], Index, RVal) :-
	SVal = [H|T],
	PVal = [U1],
	elementAt(Index, H, V),
	( B \= V ->
	  ( P = [] ->
	    Remove = 1
	  ; isRemoveValsPremises(U1, P, Var, Remove)
	  ),
	  ( Remove = 1 ->
	    RV = [H]
	  ; RV = []
	  )
	; RV = []
	),
	isRemoveVals(PVal, T, B, Var, [P|C], Index, RVal1),
	append(RV, RVal1, RVal).


%% isRemoveValsPremises(+Values, +Premises, -Remove)
isRemoveValsPremises(_, [], _, 2).
isRemoveValsPremises(Val, [H|T], Var, Remove) :-
	isRemoveValsPremises(Val, T, Var, Remove1),
	determineB(H, BP),
	isConnectives(H, BP, Var1, Con),
%	isMainConnective(H, Con, BP),
%	nl, writef('    MC: H %q -- con %q -- BP %q -- Var1 %q / Var %q', [H, Con, BP, Var1, Var]),
	determineRemove(Val, BP, Con, Var, Var1, Remove2),
%	nl, writef('    Remove? %q', [Remove2]),
	( Remove1 = 2 ->
	  Remove = Remove2
	; ( (Remove1 = 0; Remove2 = 0) ->
	    Remove = 0
	  ; Remove = 1
	  )
	). 
%% isRemoveValsPremises(+Values, +Premises, -Remove)
isRemoveValsPremisesBinary(_, _, [], _, _, 2).
isRemoveValsPremisesBinary(ValU1, ValU2, [H|T], VarU1, VarU2, Remove) :-
	isRemoveValsPremisesBinary(ValU1, ValU2, T, VarU1, VarU2, Remove1),
	determineB(H, BP),
	isConnectives(H, BP, Var, Con),
%	isMainConnective(H, Con, BP),
%	nl, writef('    MC: H %q -- con %q -- BP %q -- U1 %q (VarU1 %q), U2 %q (VarU2 %q) / Var %q', [H, Con, BP, ValU1, VarU1, ValU2, VarU2, Var]),
	( member(VarU1, Var) ->
	  determineRemove(ValU1, BP, Con, Var, Var1, Remove2)
	; ( member(VarU2, Var) ->
	    determineRemove(ValU2, BP, Con, Var, Var1, Remove2)
	  ; Remove2 = 2
	  )
	),
%	nl, writef('    Remove? %q', [Remove2]),
	( Remove1 = 2 ->
	  Remove = Remove2
	; ( (Remove1 = 0; Remove2 = 0) ->
	    Remove = 0
	  ; Remove = 1
	  )
	). 


%% determineRemove(+Value, +B, +Connectives, -Remove)
determineRemove(_, _, [], _, [], 0).
determineRemove(Val, B, [''|T], Var, [H|T1], Remove) :-
	determineRemove(Val, B, T, Var, T1, Remove1),
	( Remove1 = 1 ->
	  Remove = 1
	; elementAt(0, Val, V),
%	  nl, writef('        B %q = V %q ; Var %q = H %q', [B, V, Var, H]),
	  ( Var = H ->
            ( B = V ->
	      Remove = 1
	    ; Remove = 0
	    )
	  ; Remove = 2
	  )
	).
determineRemove(Val, B, ['*1'|T], Var, [H|T1], Remove) :-
	determineRemove(Val, B, T, Var, T1, Remove1),
	( Remove1 = 1 ->
	  Remove = 1
	; elementAt(1, Val, V),
%	  nl, writef('        B %q = V %q ; Var %q = H %q', [B, V, Var, H]),
	  ( Var = H ->
            ( B = V ->
	      Remove = 1
	    ; Remove = 0
	    )
	  ; Remove = 2
	  )
	).
determineRemove(Val, B, ['*2'|T], Var, [H|T1], Remove) :-
	determineRemove(Val, B, T, Var, T1, Remove1),
	( Remove1 = 1 ->
	  Remove = 1
	; elementAt(2, Val, V),
%	  nl, writef('        B %q = V %q ; Var %q = H %q', [B, V, Var, H]),
	  ( Var = H ->
            ( B = V ->
	      Remove = 1
	    ; Remove = 0
	    )
	  ; Remove = 2
	  )
	).

removeNoSubFormula(_, [], []).
removeNoSubFormula(Var, [[A,C]|T], PremisesR) :-
	( (member(Var, A) ; member('*1' Var, A) ; member('*2' Var, A)) ->
	  Append = [[A, C]]
	; ( (member(Var, C);  member('*1' Var, C) ; member('*2' Var, C))  ->
	  Append = [[A, C]]
	  ; Append = []
	)),
	removeNoSubFormula(Var, T, PremisesR1),
	append(Append, PremisesR1, PremisesR).

%% isConnectives (+Premise, +B, -Connective)
%% 	determines whether the premises contain the connectives in the antecedent
%% 	or the succedent and which one
isConnectives([A, S], B, Vars, Connective) :-
	( B = 0 ->
	  isConnective(A, V, C)
	; isConnective(S, V, C)
	),
	Connective=C,
	Vars = V.

%% isConnectives (+Component, -Connective)
%% 	determines the connective
isConnective([], [], []).
isConnective([('*1' X)|T], V, C) :-
	isConnective(T, V1, C1),
	append(['*1'], C1, C),
	append([X], V1, V).
isConnective([('*2' X)|T], V, C) :-
	isConnective(T, V1, C1),
	append(['*2'], C1, C),
	append([X], V1, V).
isConnective([X|T], V, C) :-
	isConnective(T, V1, C1),
	append([''], C1, C),
	append([X], V1, V).

isMainConnective([A, _], '*1', X, 0) :-
	member(('*1' X), A),
	X \= ('*1' _),
	X \= ('*2' _),
	X \= (_ v _),
	X \= (_ -> _),
	X \= (_ & _).
isMainConnective([A, _], '*2', X, 0) :-
	member(('*2' X), A),
	X \= ('*1' _),
	X \= ('*2' _),
	X \= (_ v _),
	X \= (_ -> _),
	X \= (_ & _).
isMainConnective([_, S], '*1', X, 1) :-
	member(('*1' X), S),
	X \= ('*1' _),
	X \= ('*2' _),
	X \= (_ v _),
	X \= (_ -> _),
	X \= (_ & _).
isMainConnective([_, S], '*2', X, 1) :-
	member(('*2' X), S),
	X \= ('*1' _),
	X \= ('*2' _),
	X \= (_ v _),
	X \= (_ -> _),
	X \= (_ & _).
isMainConnective(_,_,_,2).

isMainUConnective([A, _], '*1', X, 0) :-
	member(('*1'('*1' X)), A).
isMainUConnective([A, _], '*2', X, 0) :-
	member(('*1'('*2' X)), A).
isMainUConnective([A, _], '*1', X, 0) :-
	member(('*2'('*1' X)), A).
isMainUConnective([A, _], '*2', X, 0) :-
	member(('*2'('*2' X)), A).
isMainUConnective([_, S], '*1', X, 1) :-
	member(('*1'('*1' X)), S).
isMainUConnective([_, S], '*2', X, 1) :-
	member(('*1'('*2' X)), S).
isMainUConnective([_, S], '*1', X, 1) :-
	member(('*2'('*1' X)), S).
isMainUConnective([_, S], '*2', X, 1) :-
	member(('*2'('*2' X)), S).
isMainUConnective(_,'',_,2).

isMainBConnective([A, _], 'v', '*1', X, Y, 0) :-
	member(('*1'(X v Y)), A).
isMainBConnective([A, _], 'v', '*2', X, Y, 0) :-
	member(('*2'(X v Y)), A).
isMainBConnective([_, A], 'v', '*1', X, Y, 1) :-
	member(('*1'(X v Y)), A).
isMainBConnective([_, A], 'v', '*2', X, Y, 1) :-
	member(('*2'(X v Y)), A).
isMainBConnective([A, _], '->', '*1', X, Y, 0) :-
	member(('*1'(X -> Y)), A).
isMainBConnective([A, _], '->', '*2', X, Y, 0) :-
	member(('*2'(X -> Y)), A).
isMainBConnective([_, A], '->', '*1', X, Y, 1) :-
	member(('*1'(X -> Y)), A).
isMainBConnective([_, A], '->', '*2', X, Y, 1) :-
	member(('*2'(X -> Y)), A).
isMainBConnective([A, _], '&', '*1', X, Y, 0) :-
	member(('*1'(X & Y)), A).
isMainBConnective([A, _], '&', '*2', X, Y, 0) :-
	member(('*2'(X & Y)), A).
isMainBConnective([_, A], '&', '*1', X, Y, 1) :-
	member(('*1'(X & Y)), A).
isMainBConnective([_, A], '&', '*2', X, Y, 1) :-
	member(('*2'(X & Y)), A).
isMainBConnective(_,_,'',_,_,2).

