%%% Filename : p_tex.pl
%%% Date     : 16.03.2012
%%% Contents : module for writing the output into a tex file
%%% Author   : Lara Katharina Spendier <lara@logic.at>

%% texOut(+Axiom, +Rules, +Matrices)
%%	Axiom 	    ... Axiom the user provided
%% 	Rules  	    ... Analytic Rules for the axiom ([[Premises, Conclusion],...])
%%	Matrices    ... PNMatrices / Truth Tables
texOut(100,_, _) :-
	telling(Old),
	tell('ptex/Paralyzer.sty'),
	write('No'),
	told,
	tell('pencoding/ParalyzerEncoding.thy'),
	write('No'),
	told,
	tell(Old).

texOut(TAxiom, Rules, Matrices, UPredef, NumberAxioms) :-
	Matrices = [_, TVal, And, Or, Implies, Star1, Star2],
	telling(Old),
	tell('ptex/Paralyzer.sty'), 
	( NumberAxioms > 1 ->
	  print_phrase(texAxiomNoun("axioms")),
	  print_phrase(texAxiomVerb("are"))
	; print_phrase(texAxiomNoun("axiom")),
	  print_phrase(texAxiomVerb("is"))
	),
	length(Rules, N),
	( N = 1 ->
	  print_phrase(texRule("rule"))
	; print_phrase(texRule("rules"))
	),
	length(TVal, TN),
	( TN > 4 ->
	  print_phrase(texTruthTables("TruthTablesBig"))
	; print_phrase(texTruthTables("TruthTablesSmall"))
	),
	( UPredef = 0 ->
	  print_phrase(texNameBaseCalculus("LK$^+$")),
	  print_phrase(texNameBaseLogic("HCL$^+$")),
	  print_phrase(texBaseCalculusTab("CalculusLK")),
	  print_phrase(texBaseCalculusDesc("CalculusDescLK")),
	  print_phrase(texDescBaseLogic("DescLogicLK")),
	  ( Star2 = [] ->
	    print_phrase(texConnectiveNoun),
	    print_phrase(texUnaryConnective)
	  ; print_phrase(texConnectiveNouns),
	    print_phrase(texUnaryConnectives)
	  )
	; print_phrase(texNameBaseCalculus("system for BK")),
	  print_phrase(texNameBaseLogic("BK")),
	  print_phrase(texBaseCalculusTab("CalculusBK")),
	  print_phrase(texBaseCalculusDesc("CalculusDescBK")),
	  print_phrase(texDescBaseLogic("DescLogicBK")),
	  print_phrase(texConnectiveNouns),
	  print_phrase(texUnaryConnectives)
	),
	nl, print_phrase(texNewAxiom(TAxiom)),
	nl, print_phrase(texNewAxiomNotMath(TAxiom)),
	nl, print_phrase(texNewRuleStart),
	createSeqRules(Rules),
	print_phrase(texNewRuleEnd),
	texOutVM(TVal, STVal),
	print_phrase(texVM(STVal)),
	texOutDM(TVal, DTVal),
	print_phrase(texDM(DTVal)),
	texOutUnaryTT(Star1, SStar1),
	print_phrase(texStar1TT(SStar1)),
	( Star2 \= [] ->
	  texOutUnaryTT(Star2, SStar2),
	  print_phrase(texStar2TT(SStar2))
	),
	length(TVal, TupleCount),
	%% AND %%
	texOutBinaryHeader(TVal, TupleCount, AndHeader),
  	texAddColumns(TupleCount, ColumnsAnd),
	string_concat(ColumnsAnd, '} \\widetilde{\\wedge} ', CA1),
  	string_concat(CA1, AndHeader, CA2),
	string_concat(CA2, ' \\\\ \\hline \\hline ', SAndHeader),
	texOutBinaryTT(And, SAndRows, IndAnd),
	texTuple2(IndAnd, IndAnd1),
%        string_concat('\\tup{', IndAnd1, SAR1),
%	string_concat(SAR1, '} ', SAR2),
	SAR2 = IndAnd1,
	string_concat(SAR2, SAndRows, SAR),
	string_concat(SAndHeader, SAR, SAnd),
	nl,print_phrase(texAndTT(SAnd)),
	%% OR %%
	texOutBinaryHeader(TVal, TupleCount, OrHeader),
 	texAddColumns(TupleCount, ColumnsOr),
	string_concat(ColumnsOr, '} \\widetilde{\\vee} ', CO1),
 	string_concat(CO1, OrHeader, CO2),
	string_concat(CO2, ' \\\\ \\hline \\hline ', SOrHeader),
	texOutBinaryTT(Or, SOrRows, IndOr),
	texTuple2(IndOr, IndOr1),
%        string_concat('\\tup{', IndOr1, SOR1),
%	string_concat(SOR1, '} ', SOR2),
	SOR2 = IndOr1,
	string_concat(SOR2, SOrRows, SOR),
	string_concat(SOrHeader, SOR, SOr),
	nl,print_phrase(texOrTT(SOr)),
	%% IMPLIES %%
	texOutBinaryHeader(TVal, TupleCount, ImpHeader),
 	texAddColumns(TupleCount, ColumnsImp),
	string_concat(ColumnsImp, '} \\widetilde{\\su} ', CI1),
 	string_concat(CI1, ImpHeader, CI2),
	string_concat(CI2, ' \\\\ \\hline \\hline ', SImpHeader),
	texOutBinaryTT(Implies, SImpRows, IndImp),
	texTuple2(IndImp, IndImp1),
%        string_concat('\\tup{', IndImp1, SIR1),
%	string_concat(SIR1, '} ', SIR2),
	SIR2 = IndImp1,
	string_concat(SIR2, SImpRows, SIR),
	string_concat(SImpHeader, SIR, SImp),
	nl,print_phrase(texImpliesTT(SImp)),
	checkEmptySpots(Matrices,EmptySpot),
	( EmptySpot = 1 ->
	  print_phrase(texAnalyticity("NoAnalyticity")),
	  print_phrase(texArticleAnalytic("a")),
	  print_phrase(texIntro("IntroNotAnalytic")),
	  print_phrase(texTitle("A sequent "))
	; print_phrase(texAnalyticity("OKAnalyticity")),
	  print_phrase(texArticleAnalytic("an analytic")),
	  print_phrase(texIntro("IntroAnalytic")),
	  print_phrase(texTitle("An analytic "))
	),
	told,
	tell(Old).

%% createSeqRule(+[Rules])
%% 	[Rules]  ... Several analytic rules for the class N2
createSeqRules([]).
createSeqRules([[]|T]) :-
	createSeqRules(T).
%createSeqRules([[Prem, Con]|T]) :-
createSeqRules([H|T]) :-
	H = [[Prem, Con]|T1],
	createSeqRule(Prem, Con),
	createSeqRules([T1|T]).

%% createSeqRule(+Prem, +Con)
%% 	Prem,Con  ... Logical Sequent Rule for the axiom (Premises, Conclusion)
createSeqRule(Prem, Con) :-
	length(Prem,N),
	member(N, [0,1]),
	rule2tex(Prem, TPrem, []),
	rule2tex(Con, TCon, []),
	print_phrase(texNewAxC(TPrem)),
	print_phrase(texNewUnCon(TCon)).

createSeqRule(Prem, Con) :-
	length(Prem,N),
	member(N, [2,4]),
	createPremises(Prem, TPrem),
	N1 is N/2+1,
	printBinPremises(0, TPrem, N1, 1),
	rule2tex(Con, TCon, []),
	print_phrase(texNewBinCon(TCon)).

createSeqRule(Prem, Con) :-
	length(Prem,N),
	createPremises(Prem, TPrem),
	N1 is N//3,
	N2 is N mod 3,
	printTriPremises(0, TPrem, N1, N2, 1),
	rule2tex(Con, TCon, []),
	print_phrase(texNewTriCon(TCon)).

%% createPremises(+Prem,-TPrem])
%% 	Prem  ... Premises of the rule computed by AxiomCalc
%%	TPrem ... Premises of the rule in tex-format
createPremises([], []).
createPremises([H|T], TPrem) :-
	rule2tex(H, TH, []),
	createPremises(T, TPrem1),
	append([TH],TPrem1,TPrem).

%% printBinPremises(+Hyper, +Prem, +N, +I])
%%	for (hyper)sequent rules that have 2 or 4 premises
%% 	Hyper ... is 1 if hypersequent, 0 if sequent
%%	Prem  ... Premises of the rule
%%	N     ... number of premises
%%	I     ... counter
printBinPremises(_, [], _, _).

printBinPremises(Hyper, [H|T], N, I) :-
	member(I, [1,N]),
	I1 is I+1,
	( Hyper = 1 ->
	  append("G\\hh",H,H1)
	; H1 = H
	),
	print_phrase(texNewAxC(H1)),
	printBinPremises(Hyper, T, N, I1).
	
printBinPremises(Hyper, [H|T], N, I) :-
	I1 is I+1,
	( Hyper = 1 ->
	  append("G\\hh",H,H1)
	; H1 = H
	),
	print_phrase(texNoLine),
	print_phrase(texNewUnC(H1)),
	printBinPremises(Hyper, T, N, I1).

%% printTriPremises(+Hyper, +Prem, +Div, +Mod, +I])
%%	for (hyper)sequent rules that have 3 or >4 premises
%% 	Hyper ... is 1 if hypersequent, 0 if sequent
%%	Prem  ... Premises of the rule
%%	Div   ... number of premises / 3 (Integer division)
%%	Mod   ... number of premises mod 3
%%		  --> to balance the number of premises in the rule!
%%	I     ... counter
printTriPremises(_, [], _, _, _).

printTriPremises(Hyper, [H|T], Div, Mod, I) :-
	( Mod > 0 ->
 	  M1 is 1+Div+1
	; M1 is 1+Div
	), 
	( Mod > 1 ->
	  M2 is M1+Div+1
	; M2 is M1+Div
	),
	member(I, [1,M1,M2]),
	I1 is I+1,
	( Hyper = 1 ->
	  append("G\\hh",H,H1)
	; H1 = H
	),
	print_phrase(texNewAxC(H1)),
	printTriPremises(Hyper, T, Div, Mod, I1).
	
printTriPremises(Hyper, [H|T], Div, Mod, I) :-
	I1 is I+1,
	( Hyper = 1 ->
	  append("G\\hh",H,H1)
	; H1 = H
	),
	print_phrase(texNoLine),
	print_phrase(texNewUnC(H1)),
	printTriPremises(Hyper, T, Div, Mod, I1).
