(*  Title:      Sequents/ParalyzerEncoding.thy 
Author:     Paralyzer, Vienna University of Technology
            based on LK0.thy by L. Paulson, University of Cambridge
Copyright   2013 Vienna University of Technology
*)

header {* Propositonal Paraconsistent Logic BK + 
(*2 a-> *2 (a&b);*1 a v *1 b-> *1 (a&b)) *}

theory ParalyzerEncoding
imports Sequents
begin

consts
  Trueprop       :: "two_seqi"
  conj         :: "[o,o] => o"       (infixr "&" 35)
  disj         :: "[o,o] => o"       (infixr "|" 30)
  imp          :: "[o,o] => o"       (infixr "-->" 25)
  not          :: "o => o"           ("~ _" [40] 40)
  circ         :: "o => o"           ("+ _" [39] 39)

syntax
 "_Trueprop"    :: "two_seqe" ("((_)/ |- (_))" [6,6] 5)

parse_translation {* [(@{syntax_const "_Trueprop"},
   two_seq_tr @{const_syntax Trueprop})] *}
print_translation {* [(@{const_syntax Trueprop},
   two_seq_tr' @{syntax_const "_Trueprop"})] *}

notation (xsymbols)
  conj  (infixr "<and>" 35) and
  disj  (infixr "<or>" 30) and
  imp  (infixr "<longrightarrow>" 25) 
  and not ("<not> _" [40] 40)
  and circ ("<circ> _" [39] 39)

notation (HTML output)
  conj  (infixr "<and>" 35) and
  disj  (infixr "<or>" 30)
  and not ("<not> _" [40] 40)
  and circ ("<circ> _" [39] 39)

axioms

  (*Structural rules: contraction, thinning, exchange*)

  contRS: "$H |- $E, $S, $S, $F ==> $H |- $E, $S, $F"
  contLS: "$H, $S, $S, $G |- $E ==> $H, $S, $G |- $E"

  thinRS: "$H |- $E, $F ==> $H |- $E, $S, $F"
  thinLS: "$H, $G |- $E ==> $H, $S, $G |- $E"

  exchRS: "$H |- $E, $R, $S, $F ==> $H |- $E, $S, $R, $F"
  exchLS: "$H, $R, $S, $G |- $E ==> $H, $S, $R, $G |- $E"

  cut:   "[| $H |- $E, P;  $H, P |- $E |] ==> $H |- $E"

  (*Propositional rules*)

  basic: "$H, P, $G |- $E, P, $F"

  conjR: "[| $H|- $E, P, $F;  $H|- $E, Q, $F |] ==> $H|- $E, P&Q, $F"
  conjL: "$H, P, Q, $G |- $E ==> $H, P & Q, $G |- $E"

  disjR: "$H |- $E, P, Q, $F ==> $H |- $E, P|Q, $F"
  disjL: "[| $H, P, $G |- $E;  $H, Q, $G |- $E |] ==> $H, P|Q, $G |- $E"

  impR:  "$H, P |- $E, Q, $F ==> $H |- $E, P-->Q, $F"
  impL:  "[| $H,$G |- $E,P;  $H, Q, $G |- $E |] ==> $H, P-->Q, $G |- $E"

  (** new **)
    notR:  "$H, P |- $E, $F ==> $H |- $E, ~P, $F"
  circR: "$H, P, ~P |- $E, $F ==> $H |- $E, +P, $F"
  circL: "[| $H, $G |- $E, P; $H, $G |- $E, ~P |] ==> $H, +P, $G |- $E"

ruleStar1: "[| $G,~P,$H |- $E; $G, $H |- $E, P; $G, $H |- $E, Q|] ==> $G,~(P&Q),$H |- $E"
ruleStar2: "$G |- $E, ~P,$F ==> $G |- $E, ~(P&Q),$F"
ruleStar3: "$G |- $E, ~Q,$F ==> $G |- $E, ~(P&Q),$F"


(** Structural Rules on formulas **)

(*contraction*)

lemma contR: "$H |- $E, P, P, $F ==> $H |- $E, P, $F"
  by (rule contRS)

lemma contL: "$H, P, P, $G |- $E ==> $H, P, $G |- $E"
  by (rule contLS)

(*thinning*)

lemma thinR: "$H |- $E, $F ==> $H |- $E, P, $F"
  by (rule thinRS)

lemma thinL: "$H, $G |- $E ==> $H, P, $G |- $E"
  by (rule thinLS)

(*exchange*)

lemma exchR: "$H |- $E, Q, P, $F ==> $H |- $E, P, Q, $F"
  by (rule exchRS)

lemma exchL: "$H, Q, P, $G |- $E ==> $H, P, Q, $G |- $E"
  by (rule exchLS)

(*Tactic*)

ML {*
    val apply_tac =
    let
      val rules = @{thms basic  conjL conjR disjL disjR impL impR 
       notR circR circL  ruleStar1 ruleStar2 ruleStar3}
    in
      atac
      ORELSE' resolve_tac rules
    end
*}
end